@@ -1,15 +1,21 @@
+import multiprocessing
 import os
-from multiprocessing import Pool
+import sys
 
 import numpy as np
-from matplotlib import pyplot as plt
-from scipy.interpolate import interp1d
 from tqdm import tqdm
 
 import amical
 from amical.analysis import models
 from amical.dpfit import leastsqFit
-from amical.tools import mas2rad, roundSciDigit
+from amical.tools import mas2rad
+from amical.tools import roundSciDigit
+
+if sys.platform == "darwin":
+    multiprocessing.set_start_method(
+        "fork", force=True
+    )  # this fixes loop in python 3.8 on MacOS
+
 
 err_pts_style = {
     "linestyle": "None",
@@ -33,9 +39,7 @@ def select_model(name):
     elif name == "binary_res":
         model = models.visBinary_res
     elif name == "edisk":
-        model = models.visEllipticakUniformDisk
-    elif name == "debrisDisk":
-        model = models.visDebrisDisk
+        model = models.visEllipticalDisk
     elif name == "clumpyDebrisDisk":
         model = models.visClumpDebrisDisk
     else:
@@ -49,10 +53,10 @@ def check_params_model(param):
     isValid = True
     log = ""
     if param["model"] == "edisk":
-        elong = param["elong"]
-        minorAxis = mas2rad(param["minorAxis"])
-        angle = np.deg2rad(param["angle"])
-        if (elong < 1) or (angle < 0) or (angle > np.pi) or (minorAxis < 0):
+        elong = np.cos(np.deg2rad(param["incl"]))
+        majorAxis = mas2rad(param["majorAxis"])
+        posang = np.deg2rad(param["posang"])
+        if (elong < 1) or (posang < 0) or (posang > 180) or (majorAxis < 0):
             log = "# elong > 1,\n# minorAxis > 0 mas,\n# 0 < angle < 180 deg.\n"
             isValid = False
 
@@ -151,7 +155,7 @@ def model_parallelized(obs, param, ncore=12):
     """
     Compute model for each data points in obs tuple (multiprocess version).
     """
-    pool = Pool(ncore)
+    pool = multiprocessing.Pool(ncore)
 
     b = np.array([param] * len(obs))
     c = b.reshape(len(obs), 1)
@@ -222,17 +226,17 @@ def fits2obs(
     ncp = data.cp.shape[0]
 
     vis2_data = data.vis2.flatten()  # * 0.97
-    e_vis2_data = (data.e_vis2.flatten() ** 2 + extra_error_v2 ** 2) ** 0.5
+    e_vis2_data = (data.e_vis2.flatten() ** 2 + extra_error_v2**2) ** 0.5
     flag_V2 = data.flag_vis.flatten()
 
     if input_rad:
         cp_data = np.rad2deg(data.cp.flatten())
         e_cp_data = np.rad2deg(
-            np.sqrt(data.e_cp.flatten() ** 2 + extra_error_cp ** 2) * err_scale
+            np.sqrt(data.e_cp.flatten() ** 2 + extra_error_cp**2) * err_scale
         )
     else:
         cp_data = data.cp.flatten()
-        e_cp_data = np.sqrt(data.e_cp.flatten() ** 2 + extra_error_cp ** 2) * err_scale
+        e_cp_data = np.sqrt(data.e_cp.flatten() ** 2 + extra_error_cp**2) * err_scale
 
     flag_CP = data.flag_cp.flatten()
 
@@ -354,7 +358,7 @@ def fits2obs(
 
     N_cp_rest = len(obs) - N_v2_rest
 
-    Obs = np.array(obs)
+    Obs = np.array(obs, dtype=object)
 
     if verbose:
         print(
@@ -369,7 +373,7 @@ def fits2obs(
                 % (wl_min, chr(955), wl_max)
             )
         if cond_uncer:
-            print(fr"-> Restriction on uncertainties: {chr(949)} < {rel_max:2.1f} %")
+            print(rf"-> Restriction on uncertainties: {chr(949)} < {rel_max:2.1f} %")
 
     return Obs
 
@@ -440,6 +444,8 @@ def compute_chi2_curve(
     `errors_chi2` {float}:
         Computed errors using the chi2 curve at the position of the chi2_r.min() + 1.
     """
+    import matplotlib.pyplot as plt
+    from scipy.interpolate import interp1d
 
     fit = smartfit(
         obs,
@@ -533,7 +539,7 @@ def compute_chi2_curve(
         ymin=ymin,
         ymax=ymax,
         color="#dbe4e8",
-        label=fr"$\sigma_{{m1}}=$-{dr1_r}/+{dr2_r}",
+        label=rf"$\sigma_{{m1}}=$-{dr1_r}/+{dr2_r}",
     )
     plt.axvspan(
         fitted_param - fit_e_theta,
@@ -596,11 +602,12 @@ def plot_model(
         Limits of the y-axis,\n
 
     """
+    import matplotlib.pyplot as plt
 
     d = amical.loadc(inputdata)
 
-    e_vis2 = np.sqrt(d.e_vis2 ** 2 + extra_error_v2 ** 2)
-    e_cp = np.sqrt(d.e_cp ** 2 + extra_error_cp ** 2) * err_scale
+    e_vis2 = np.sqrt(d.e_vis2**2 + extra_error_v2**2)
+    e_cp = np.sqrt(d.e_cp**2 + extra_error_cp**2) * err_scale
 
     model_target = select_model(param["model"])
 
@@ -706,6 +713,8 @@ def plot_model(
     axd["res_cp"].set_xlabel("Sp. Freq. [%s]" % label_unit[unit])
 
     if save:
+        import matplotlib.pyplot as plt
+
         filename = os.path.basename(inputdata) + "_fit_candid.pdf"
         if outputfile is not None:
             filename = outputfile
@@ -764,7 +773,7 @@ def smartfit(
 
     errs = [o[-1] for o in obs]
     if normalizeErrors:
-        errs = _normalize_err_obs(obs)
+        errs = _normalize_err_obs(obs, verbose=False)
 
     # -- avoid fitting string parameters
     tmp = list(filter(lambda x: isinstance(first_guess[x], str), first_guess.keys()))
