@@ -1,13 +1,15 @@
+import multiprocessing
 import pickle
+import sys
 import time
-from multiprocessing import Pool
 
-import corner
-import matplotlib.pyplot as plt
 import numpy as np
-import scipy.interpolate as interp
-from scipy.optimize import leastsq
-from tqdm import tqdm
+
+if sys.platform == "darwin":
+    multiprocessing.set_start_method(
+        "fork", force=True
+    )  # this fixes loop in python 3.8 on MacOS
+
 
 """------------------------------------------------------------------------
 cp_tools.py - a collection of functions useful for closure phase analysis
@@ -195,6 +197,8 @@ def cp_model(params, u, v, wavels, model="constant"):
             ndof (the wavelength channels are evenly spaced cubic interpolations in params[2:])
             polynomial (of the form Sum[n] params[n+2]*(wavelength*1e6)**n )
     NOTE: This doesn't allow for nonzero size of each component!"""
+    import scipy.interpolate as interp
+
     nwav = wavels.size
     model_params = np.zeros(nwav + 2)
     model_params[0:2] = params[0:2]
@@ -218,7 +222,7 @@ def cp_model(params, u, v, wavels, model="constant"):
         cons = np.repeat(0.0, nwav)
         xax = (wavels - np.min(wavels)) / (np.max(wavels) - np.min(wavels))
         for order in range(ndof):
-            cons += coefficients[order] * xax ** order
+            cons += coefficients[order] * xax**order
     else:
         raise NameError("Unknown model input to cp_model")
     model_params[2:] = cons
@@ -297,7 +301,7 @@ def hammer(
     #    p0 = [ivar + 0.75*ivar*np.random.rand(ndim) for i in range(nwalcps)] # initialise walcps in a ball
     # print('\n -- Running emcee --')
 
-    t3err = np.sqrt(cpo.t3err ** 2 + extra_error ** 2)
+    t3err = np.sqrt(cpo.t3err**2 + extra_error**2)
     t3err *= err_scale
 
     t0 = time.time()
@@ -318,7 +322,7 @@ def hammer(
             threads=threads,
         )
     else:
-        proj_t3err = np.sqrt(cpo.proj_t3err ** 2 + extra_error ** 2)
+        proj_t3err = np.sqrt(cpo.proj_t3err**2 + extra_error**2)
         proj_t3err *= err_scale
         sampler = emcee.EnsembleSampler(
             nwalcps,
@@ -342,11 +346,7 @@ def hammer(
     if verbose:
         print("Time elapsed =", tf - t0, "s")
 
-    chain = sampler.flatchain
-
-    # Remove the burn in
-    chain = chain[burn_in:]
-
+    chain = sampler.get_chain(discard=burn_in, thin=15, flat=True)
     seps = chain[:, 0]
     ths = chain[:, 1]
     cs = chain[:, 2:][:, 0]
@@ -440,6 +440,8 @@ def hammer(
         # plt.ylabel('CR')
         # plt.tight_layout()
         # plt.show(block=False)
+        import matplotlib.pyplot as plt
+        import corner
 
         fig = corner.corner(
             chain,
@@ -473,7 +475,7 @@ def hammer(
         "delpa": dth,
         "con": meanc,
         "delcon": dc,
-        "chain": sampler.chain,
+        "chain": sampler.get_chain(),
     }
     data2 = {
         "sep": res_p[0],
@@ -485,152 +487,12 @@ def hammer(
         "cr": res_p[2],
         "delcrm": err_m[2],
         "delcrp": err_p[2],
-        "chain": sampler.chain,
+        "chain": sampler.get_chain(),
     }
 
-    # and clean up
-    # if threads > 1:
-    #     sampler.pool.terminate()
     return data, data2
 
 
-# =========================================================================
-# =========================================================================
-
-
-# def nest(cpo, paramlimits=[20., 250., 0., 360., 1.0001, 10], resume=False, eff=0.3, multi=True,
-#          err_scale=1., extra_error=0., plot=True, npoints=1000, logcrat_prior=True,
-#          use_cov=False, projected=False, model='constant'):
-#     '''Default implementation of a MultiNest fitting routine for closure
-#     phase data. Requires a closure phase cpo object, parameter limits and
-#     sensible keyword arguments for the multinest parameters.
-
-#     This function does very naughty things creating functions inside this
-#     function because PyMultiNest is very picky about how you pass it
-#     data.
-
-#     Optional parameter eff tunes sampling efficiency, and multi toggles multimodal
-#     nested sampling on and off. Turning off multimodal sampling results in a speed
-#     boost of ~ 20-30%.
-
-#     logcrat_prior allows the prior for the contrast ratios to be flat in logarithmic space
-
-#     paramlimits is needed here, and must be a list with length = ncomp,
-#     with paramlimits[companion_number]=[minsep,maxsep,minpa,maxpa,mincrat,maxcrat]
-#     '''
-#     import pymultinest
-
-#     if projected:
-#         proj_t3err = np.sqrt(cpo.proj_t3err**2 + extra_error**2)
-#         proj_t3err *= err_scale
-#     else:
-#         t3err = np.sqrt(cpo.t3err**2+extra_error**2)
-#         t3err *= err_scale
-
-#     if logcrat_prior:
-#         def myprior(cube, ndim, n_params, paramlimits=paramlimits):
-#             cube[0] = cube[0]*(paramlimits[1] - paramlimits[0])+paramlimits[0]
-#             cube[1] = cube[1]*(paramlimits[3] - paramlimits[2])+paramlimits[2]
-#             for ix in range(n_params-2):
-#                 cube[ix+2] = 10**(cube[ix+2]*(np.log10(paramlimits[5]) -
-#                                               np.log10(paramlimits[4]))+np.log10(paramlimits[4]))
-#     else:
-#         def myprior(cube, ndim, n_params, paramlimits=paramlimits):
-#             cube[0] = cube[0]*(paramlimits[1] - paramlimits[0])+paramlimits[0]
-#             cube[1] = cube[1]*(paramlimits[3] - paramlimits[2])+paramlimits[2]
-#             for ix in range(n_params-2):
-#                 cube[ix+2] = cube[ix+2] * \
-#                     (paramlimits[5] - paramlimits[4])+paramlimits[4]
-
-#     if projected:
-#         def myloglike(cube, ndim, n_params):
-#             loglike = cp_loglikelihood_proj(cube[0:n_params], cpo.u, cpo.v, cpo.wavel,
-#                                             cpo.proj_t3data, proj_t3err, cpo.proj, model=model)
-#             return loglike
-
-#     elif use_cov:
-#         def myloglike(cube, ndim, n_params):
-#             loglike = cp_loglikelihood_cov(cube[0:n_params], cpo.u, cpo.v, cpo.wavel,
-#                                            cpo.t3data, cpo.cov_inv, model=model)
-#             return loglike
-
-#     else:
-#         def myloglike(cube, ndim, n_params):
-#             loglike = cp_loglikelihood(
-#                 cube[0:n_params], cpo.u, cpo.v, cpo.wavel, cpo.t3data, t3err, model=model)
-#             return loglike
-
-#     # How many parameters?
-#     if model == 'constant':
-#         parameters = ['Separation', 'Position Angle', 'Contrast Ratio']
-#     elif model == 'free':
-#         parameters = ['Separation', 'Position Angle']
-#         parameters.extend(len(cpo.wavel)*['Contrast Ratio'])
-#     else:
-#         raise Exception('Model not yet implemented in nest!')
-
-#     n_params = len(parameters)
-#     ndim = n_params
-
-#     # Check that the "chains" directory exists (which multinest needs)
-#     if os.path.exists(os.getcwd()+'/chains/') is False:
-#         os.mkdir(os.getcwd()+'/chains/')
-
-#     tic = time.time()  # start timing
-
-#     # ---------------------------------
-#     # now run MultiNest!
-#     # ---------------------------------
-
-#     pymultinest.run(myloglike, myprior, n_params, wrapped_params=[1], resume=resume, verbose=True,
-#                     sampling_efficiency=eff, multimodal=multi, n_iter_before_update=1000,
-#                     n_live_points=npoints)
-#     # let's analyse the results
-#     a = pymultinest.Analyzer(n_params=n_params)
-#     s = a.get_stats()
-
-#     toc = time.time()
-
-#     if toc-tic < 60.:
-#         print('Time elapsed =', toc-tic, 's')
-#     else:
-#         print('Time elapsed =', (toc-tic)/60., 'mins')
-
-#     print()
-#     print("-" * 30, 'ANALYSIS', "-" * 30)
-#     print("Global Evidence:\n\t%.15e +- %.15e" %
-#           (s['global evidence'], s['global evidence error']))
-#     print('')
-
-#     params = s['marginals']
-#     print_line = "{0}: {1:.3F} pm {2:.3F}"
-
-#     for param_ix in range(n_params):
-#         print(print_line.format(parameters[param_ix], params[param_ix]['median'],
-#                                 params[param_ix]['sigma']))
-#     if plot:
-#         p = pymultinest.PlotMarginalModes(a)
-#         plt.figure(figsize=(5*n_params, 5*n_params))
-#         for i in range(n_params):
-#             plt.subplot(n_params, n_params, n_params * i + i + 1)
-#             p.plot_marginal(i, with_ellipses=True,
-#                             with_points=False, grid_points=20)
-#             plt.ylabel("Probability")
-#             plt.xlabel(parameters[i])
-
-#             for j in range(i):
-#                 plt.subplot(n_params, n_params, n_params * j + i + 1)
-#                 # plt.subplots_adjust(left=0, bottom=0, right=0, top=0, wspace=0, hspace=0)
-#                 p.plot_conditional(i, j, with_ellipses=False, with_points=True, grid_points=20)
-#                 plt.xlabel(parameters[i])
-#                 plt.ylabel(parameters[j])
-
-#     return [a, s]
-
-# =========================================================================
-# =========================================================================
-
-
 def detec_sim_loopfit(everything):
     """Function for multiprocessing in detec_limits. Takes a
     single separation and full angle, contrast lists.
@@ -660,7 +522,7 @@ def detec_sim_loopfit(everything):
             #  i.e. using rnd_cp for the single star and rnd_cp-bin_cp for the binary
             #  but this simplifies to the following equation
             chi2_diff = np.sum(
-                (resids ** 2 - rnd_cp ** 2) / everything["error"][..., np.newaxis] ** 2,
+                (resids**2 - rnd_cp**2) / everything["error"][..., np.newaxis] ** 2,
                 axis=tuple(range(ndim)),
             )
 
@@ -741,7 +603,7 @@ def detec_sim_loopfit_proj(everything):
             #  i.e. using rnd_cp for the single star and rnd_cp-bin_cp for the binary
             #  but this simplifies to the following equation
             chi2_diff = np.sum(
-                (proj_resids ** 2 - proj_rnd_cp ** 2)
+                (proj_resids**2 - proj_rnd_cp**2)
                 / everything["error"][..., np.newaxis] ** 2,
                 axis=tuple(range(ndim)),
             )
@@ -815,18 +677,18 @@ def detec_limits(
     # ------------------------
     if projected is True:
         proj = cpo.proj
-        error = np.sqrt(cpo.proj_t3err ** 2 + extra_error ** 2) * err_scale
+        error = np.sqrt(cpo.proj_t3err**2 + extra_error**2) * err_scale
         n_clps = cpo.proj.shape[-1]
         n_runs = cpo.n_runs
     elif icpo is True:
         proj = []
-        cpo.t3err = np.sqrt(cpo.t3err ** 2 + extra_error ** 2)
+        cpo.t3err = np.sqrt(cpo.t3err**2 + extra_error**2)
         error = cpo.t3err * err_scale
         n_clps = cpo.n_clps
         n_runs = cpo.n_runs
     else:
         proj = []
-        cpo.t3err = np.sqrt(cpo.t3err ** 2 + extra_error ** 2)
+        cpo.t3err = np.sqrt(cpo.t3err**2 + extra_error**2)
         error = cpo.t3err * err_scale
         n_clps = cpo.ndata
         n_runs = 1
@@ -845,7 +707,7 @@ def detec_limits(
     u = cpo.u
     v = cpo.v
 
-    w = np.array(np.sqrt(u ** 2 + v ** 2)) / np.median(wavel)
+    w = np.array(np.sqrt(u**2 + v**2)) / np.median(wavel)
 
     if smin == "Default":
         smin = rad2mas(1.0 / 4 / np.max(w))
@@ -930,7 +792,7 @@ def detec_limits(
                 else:
                     print("Estimated time remaining: %.2f seconds" % (remaining))
     else:
-        pool = Pool(processes=threads)
+        pool = multiprocessing.Pool(processes=threads)
         ndetec = np.zeros((nsep, nth, ncon))
         if projected:
             ndetec = pool.map(detec_sim_loopfit_proj, all_vars)
@@ -964,10 +826,12 @@ def detec_limits(
     levels = [0.5, 0.9, 0.99, 0.999]
     mycols = ("k", "k", "k", "k")
     if not no_plot:
+        import matplotlib.pyplot as plt
+
         plt.figure(0)
         plt.clf()
         contours = plt.contour(
-            ndetec, levels, colors=mycols, linewidth=2, extent=[smin, smax, cmin, cmax]
+            ndetec, levels, colors=mycols, extent=[smin, smax, cmin, cmax]
         )
         plt.clabel(contours)
         plt.contourf(seps, cons, ndetec, levels, cmap=plt.cm.bone)
@@ -1015,6 +879,7 @@ def binary_fit(cpo, p0):
     - soluce[0] : best-fit parameters
     - soluce[1] : covariance matrix
     -------------------------------------------------------------"""
+    from scipy.optimize import leastsq
 
     if np.all(cpo.t3err == 0.0):
         print("Closure phase object instance is not calibrated.\n")
@@ -1068,45 +933,6 @@ def brute_force_chi2_grid(everything):
     return chi2
 
 
-# =========================================================================
-# =========================================================================
-
-
-# def brute_force_chi2_grid_proj(everything):
-#     '''
-#     Function for multiprocessing, fills in part of the big 3d chi2 grid in a
-#     way that minimises repeated calculations (i.e. each model only calculated once).
-#     This version handles projected closure phases
-#     '''
-#     sim_cps = everything['sim_cps']
-#     sep = everything['sep']
-#     nth = everything['nth']
-#     ncon = everything['ncon']
-#     nsim = everything['nsim']
-#     n_runs = everything['n_runs']
-#     n_clps = everything['n_clps']
-#     chi2 = np.zeros((nth, ncon, nsim))
-#     for j, th in enumerate(everything['ths']):
-#         for k, con in enumerate(everything['cons']):
-#             mod_cps = cp_model([sep, th, con], everything['u'],
-#                                everything['v'], everything['wavel'])
-#             # project and take independent clps
-#             # THIS NEEDS TO BE FIXED
-#             proj_bin_cps = project_cps(mod_cps, proj)
-#             mod_cps = np.reshape(
-#                 mod_cps, (len(mod_cps)/n_runs, n_runs), order='F')
-#             proj_bin_cp = np.transpose(everything['proj']).dot(mod_cps)
-#             proj_bin_cp = proj_bin_cp[0:n_clps, :]
-#             proj_bin_cp = proj_bin_cp.ravel()
-#             for i in range(nsim):
-#                 chi2[j, k, i] = np.sum(
-#                     ((sim_cps[:, :, i]-proj_bin_cp) / everything['error'])**2)
-#     return chi2
-
-# =========================================================================
-# =========================================================================
-
-
 def lmfit(everything):
     """Unpacks a cpo and calls the proper binary L-M fitting function"""
     this_cpo = everything["sim_cpo"]
@@ -1119,258 +945,6 @@ def lmfit(everything):
 # =========================================================================
 
 
-# def brute_force_detec_limits(cpo, nsim=100, nsep=32, nth=20, ncon=32, smin='Default', smax='Default',
-#                              cmin=10., cmax=500., extra_error=0, threads=0, save=False, include_cov=False, icpo=False, projected=False,
-#                              err_scale=1., lm_fit=True):
-#     '''uses a Monte Carlo simulation to establish contrast-separation
-#     detection limits given an array of standard deviations per closure phase.
-
-#     Because different separation-contrast grid points are entirely
-#     separate, this task is embarrassingly parallel. If you want to
-#     speed up the calculation, use multiprocessing with a threads
-#     argument equal to the number of available cores.
-
-#     Make nseps a multiple of threads! This uses the cores most efficiently.
-
-#     Hyperthreading (2x processes per core) in my experience gets a ~20%
-#     improvement in speed.
-
-#     Written by F. Martinache and B. Pope.
-
-#     This version was modified by ACC to use a brute force
-#     chi2 grid that emulates the idl program binary_grid.pro.
-
-#     lm_fit option allows an extra step involving a levenberg-marquadt fit at each point,
-#     to ensure that each set of best parameters is the actual minimum and reduce the effect of
-#     the grid size.
-#     '''
-
-#     # ------------------------
-#     # first, load your data!
-#     # ------------------------
-
-#     if projected is True:
-#         proj = cpo.proj
-#         cpo.t3err = np.sqrt(cpo.t3err**2+extra_error**2)
-#         error = cpo.t3err.ravel()*err_scale
-#         n_clps = cpo.n_good
-#         n_runs = cpo.n_runs
-#     elif icpo is True:
-#         proj = []
-#         cpo.t3err = np.sqrt(cpo.t3err**2+extra_error**2)
-#         error = cpo.t3err.ravel()*err_scale
-#         n_clps = cpo.n_clps
-#         n_runs = cpo.n_runs
-#     else:
-#         proj = []
-#         cpo.t3err = np.sqrt(cpo.t3err**2+extra_error**2)
-#         error = cpo.t3err.ravel()*err_scale
-#         n_clps = cpo.ndata
-#         n_runs = 1
-
-#     nwav = cpo.wavel.size
-#     ndata = cpo.u.shape[0]
-#     u = np.repeat(np.resize(cpo.u, [ndata, 3, 1]), nwav, 2).ravel()
-#     v = np.repeat(np.resize(cpo.v, [ndata, 3, 1]), nwav, 2).ravel()
-#     wavel = np.repeat(
-#         np.repeat(np.resize(cpo.wavel, [1, 1, nwav]), ndata, 0), 3, 1).ravel()
-
-#     ndata = n_clps*n_runs
-
-#     w = np.array(np.sqrt(u**2 + v**2))/wavel
-
-#     if smin == 'Default':
-#         smin = rad2mas(1./4/np.max(w))
-
-#     if smax == 'Default':
-#         smax = rad2mas(1./np.min(w))
-
-#     # ------------------------
-#     # initialise Monte Carlo
-#     # ------------------------
-
-#     seps = smin + (smax-smin) * np.linspace(0, 1, nsep)
-#     ths = 0.0 + 360.0 * np.linspace(0, 1, nth)
-#     cons = cmin + (cmax-cmin) * np.linspace(0, 1, ncon)
-
-#     if include_cov is True:
-#         # use full multivariate gaussian with sample covariance to get rands
-#         rands = np.zeros((n_clps*n_runs, nwav, nsim))
-#         for run in range(n_runs):
-#             rands[n_clps*(run):n_clps*(run+1), 0, :] = np.transpose(
-#                 np.random.multivariate_normal(np.zeros(n_clps), cpo.sample_cov, (nsim)))
-#     else:
-#         rands = np.random.randn(ndata, nwav, nsim)
-#     error = np.reshape(error, [ndata, nwav])
-#     sim_cps = np.reshape(rands*error[:, :, np.newaxis], (ndata, nwav, nsim))
-
-#     sim_cpos = []
-#     for ix in range(nsim):
-#         this_cpo = copy.deepcopy(cpo)
-#         this_cpo.t3data = sim_cps[:, :, ix]
-#         sim_cpos.append(this_cpo)
-
-#     # ------------------------
-#     # Run Monte Carlo
-#     # ------------------------
-#     tic = time.time()  # start the clock
-#     best_params = []
-#     if threads == 0:
-#         toc = time.time()
-#         chi2 = np.zeros((nsep, nth, ncon, nsim))
-#         print('Starting big loop over separations')
-#         for ix in range(nsep):
-#             everything = {'sep': seps[ix], 'cons': cons, 'ths': ths, 'nsim': nsim,
-#                           'nsep': nsep, 'ncon': ncon, 'nth': nth, 'u': cpo.u, 'v': cpo.v,
-#                           'sim_cps': sim_cps, 'error': error, 'wavel': cpo.wavel, 'n_runs': n_runs,
-#                           'n_clps': n_clps, 'proj': proj}
-#             if projected:
-#                 chi2[ix, :, :, :] = brute_force_chi2_grid_proj(everything)
-#             else:
-#                 chi2[ix, :, :, :] = brute_force_chi2_grid(everything)
-#             if (ix % 5) == 0:
-#                 tc = time.time()
-#                 print('Done', ix, '. Time taken:', (tc-toc), 'seconds')
-#                 toc = tc
-#     else:
-#         all_vars = []
-#         print('Making big array')
-#         for ix in range(nsep):
-#             everything = {'sep': seps[ix], 'cons': cons, 'ths': ths, 'nsim': nsim,
-#                           'nsep': nsep, 'ncon': ncon, 'nth': nth, 'u': cpo.u, 'v': cpo.v,
-#                           'sim_cps': sim_cps, 'error': error, 'wavel': cpo.wavel, 'n_runs': n_runs,
-#                           'n_clps': n_clps, 'proj': proj}
-#             all_vars.append(everything)
-#         print('Made big array for loop')
-#         pool = Pool(processes=threads)
-#         if projected:
-#             chi2 = pool.map(brute_force_chi2_grid_proj, all_vars)
-#         else:
-#             chi2 = pool.map(brute_force_chi2_grid, all_vars)
-#         pool.close()
-#         chi2 = np.array(chi2)
-#     tf = time.time()
-#     if tf-tic > 60:
-#         print('Total time elapsed:', (tf-tic)/60., 'mins')
-#     elif tf-tic <= 60:
-#         print('Total time elapsed:', tf-tic, 'seconds')
-
-#     # ------------------------
-#     # Do L-M fit to get actual best fit params
-#     # ------------------------
-#     print('Finding best parameters')
-#     best_params = np.zeros((nsim, 3))
-#     best_chi2_params = np.zeros((nsim, 3))
-#     for ix in range(nsim):
-#         b_params_ix = np.where(chi2[:, :, :, ix] == np.amin(chi2[:, :, :, ix]))
-#         b_params = [seps[b_params_ix[0][0]],
-#                     ths[b_params_ix[1][0]], cons[b_params_ix[2][0]]]
-#         best_chi2_params[ix, :] = np.array(b_params)
-#     print('Found best parameters')
-
-#     if lm_fit:
-#         print('Starting L-M fits.')
-#         tic = time.time()  # start the clock
-#         if threads == 0:
-#             for ix in range(nsim):
-#                 this_cpo = sim_cpos[ix]
-#                 [new_params, cov] = binary_fit(
-#                     this_cpo, best_chi2_params[ix, :])
-#                 if ix % 50 == 0:
-#                     print('Done', ix)
-#                 best_params[ix, :] = new_params
-#         else:
-#             all_vars = []
-#             for ix in range(nsim):
-#                 # print('Making big array:',ix,'of',nsep)
-#                 everything = {
-#                     'sim_cpo': sim_cpos[ix], 'best_chi2_params': best_chi2_params[ix, :]}
-#                 all_vars.append(everything)
-#             print('Using', threads, 'cores')
-#             pool = Pool(processes=threads)
-#             best_params = pool.map(lmfit, all_vars)
-#             pool.close()
-
-#         tf = time.time()
-#         if tf-tic > 60:
-#             print('Total time elapsed:', (tf-tic)/60., 'mins')
-#         elif tf-tic <= 60:
-#             print('Total time elapsed:', tf-tic, 'seconds')
-#     else:
-#         best_params = best_chi2_params
-
-#     # ------------------------
-#     # Collect and organise
-#     # ------------------------
-
-#     ndetec = np.zeros((ncon, nsep))
-#     # nc, ns = int(ncon), int(nsep)
-
-#     # collect them
-#     for ix in range(nsim):
-#         sep = best_params[ix][0]
-#         con = best_params[ix][2]
-#         sep_ix = np.where(abs(seps-sep) == np.amin(abs(seps-sep)))
-#         con_ix = np.where(abs(cons-con) == np.amin(abs(cons-con)))
-#         ndetec[sep_ix, con_ix] += 1
-#     # Take the cumulative sum over contrast ratio at each sep
-#     cumsum_detec = ndetec.cumsum(axis=1)
-#     # turn into %
-#     maxdetec = np.amax(cumsum_detec, axis=1)
-#     ndetec = 0*cumsum_detec
-#     for ix in range(nsep):
-#         if maxdetec[ix] == 0:
-#             print('No sims for sep '+str(seps[ix])+'mas')
-#         else:
-#             print(str(maxdetec[ix])+" in "+str(seps[ix])+" bin.")
-#             ndetec[ix, :] = cumsum_detec[ix, :]/maxdetec[ix]
-
-#     ndetec = 1-ndetec
-#     # Axes were wrong way around (I blame IDL)
-#     ndetec = np.transpose(ndetec)
-
-#     # ---------------------------------------------------------------
-#     #                        contour plot!
-#     # ---------------------------------------------------------------
-#     levels = [0., 0.9, 0.99, 0.999]
-#     mycols = ('k', 'k', 'k', 'k')
-
-#     plt.figure(0)
-#     contours = plt.contour(ndetec, levels, colors=mycols, linewidth=2,
-#                            extent=[smin, smax, cmin, cmax])
-#     plt.clabel(contours)
-#     plt.contourf(seps, cons, ndetec, levels, cmap=plt.cm.bone)
-#     plt.colorbar()
-#     plt.xlabel('Separation (mas)')
-#     plt.ylabel('Contrast Ratio')
-#     plt.title('Contrast Detection Limits')
-#     plt.draw()
-#     plt.show()
-
-#     data = {'levels': levels,
-#             'seps': seps,
-#             'angles': ths,
-#             'cons': cons,
-#             'name': cpo.name,
-#             'cumsum': cumsum_detec,
-#             'best_params': best_params,
-#             'limits': ndetec}
-
-#     if save is True:
-#         file = 'limit_lowc'+cpo.name+'.pick'
-#         print(file)
-
-#         myf = open(file, 'w')
-#         pickle.dump(data, myf)
-#         myf.close()
-
-#     return data
-
-
-# =========================================================================
-# =========================================================================
-
-
 def chi2_grid(everything):
     """Function for multiprocessing, does 2d chi2 grid for coarse_grid"""
     cpo = everything["cpo"]
@@ -1476,12 +1050,12 @@ def coarse_grid(
 
     u, v = cpo.u, cpo.v
 
-    cpo.t3err = np.sqrt(cpo.t3err ** 2 + extra_error ** 2)
+    cpo.t3err = np.sqrt(cpo.t3err**2 + extra_error**2)
     cpo.t3err *= err_scale
 
     wavel = cpo.wavel
 
-    w = np.array(np.sqrt(u ** 2 + v ** 2))
+    w = np.array(np.sqrt(u**2 + v**2))
 
     if smin == "Default":
         smin = rad2mas(1.0 / 4 / np.max(w / np.min(wavel)))
@@ -1535,13 +1109,14 @@ def coarse_grid(
                 "ix": ix,
             }
             all_vars.append(everything)
-        pool = Pool(processes=threads)
+        pool = multiprocessing.Pool(processes=threads)
         if projected:
             chi2 = pool.map(chi2_grid_proj, all_vars)
         elif use_cov:
             chi2 = pool.map(chi2_grid_cov, all_vars)
         else:
             chi2 = pool.map(chi2_grid, all_vars)
+        pool.close()
     tf = time.time()
     if tf - tic > 60:
         if verbose:
@@ -1575,6 +1150,8 @@ def coarse_grid(
     #                        contour plot!
     # ---------------------------------------------------------------
     if plot:
+        import matplotlib.pyplot as plt
+
         vmax = np.max([sep_pa.max(), sep_crat.max(), pa_crat.max()])
         plt.figure(figsize=(6, 5))
         plt.subplot(2, 2, 1)
@@ -1689,7 +1266,7 @@ def multiple_companions_hammer(
         ncomp = 1
         nparam = ndim
 
-    cpo.t3err = np.sqrt(cpo.t3err ** 2 + extra_error ** 2)
+    cpo.t3err = np.sqrt(cpo.t3err**2 + extra_error**2)
     cpo.t3err *= err_scale
 
     print("Fitting for:", ncomp, "companions")
@@ -1758,6 +1335,8 @@ def multiple_companions_hammer(
         chain[:, ix * nparam + 2 : (ix + 1) * nparam] = cs[:, ix, :]
 
     if plot is True:
+        import matplotlib.pyplot as plt
+
         plt.clf()
 
     # Loop over number of companions to calculate best params and do plots
@@ -1801,6 +1380,8 @@ def multiple_companions_hammer(
             extra_dims = "None"
 
         if plot is True:
+            import matplotlib.pyplot as plt
+
             paramnames = ["Separation", "Position Angle", "Contrast"]
             paramdims = ["(mas)", "(deg)", "Ratio"]
             for ix, par in enumerate(extra_pars):
@@ -1877,7 +1458,7 @@ def hammer_spectrum(
 
     p0 = np.array(p0)
     params = np.array(params)
-    cpo.t3err = np.sqrt(cpo.t3err ** 2 + extra_error ** 2)
+    cpo.t3err = np.sqrt(cpo.t3err**2 + extra_error**2)
     cpo.t3err *= err_scale
 
     print("Running emcee now!")
@@ -1920,6 +1501,7 @@ def hammer_spectrum(
         print("Channel", ix, ":", meanc[ix], "pm", dc[ix])
 
     if plot is True:
+        import matplotlib.pyplot as plt
 
         plt.clf()
 
@@ -1969,6 +1551,8 @@ def find_extra_error(
     """Finds the extra error needed for reduced chi squared to be 1. This is
     done by trying for many values of extra error and then interpolating.
     If dof is not set, then the degrees of freedom are taken as cpo.t3data.size."""
+    import scipy.interpolate as interp
+    import matplotlib.pyplot as plt
 
     model_cps = cp_model(params, cpo.u, cpo.v, cpo.wavel, model=model)
     if projected:
@@ -1992,11 +1576,11 @@ def find_extra_error(
     for ix, err in enumerate(extra_errors):
 
         # Calculate the closure phase uncertainties for this amount of extra_error
-        t3err = np.sqrt(cp_err ** 2 + err ** 2)
+        t3err = np.sqrt(cp_err**2 + err**2)
         t3err *= err_scale
 
-        chis_bin[ix] = np.sum(cp_resids ** 2 / t3err ** 2)
-        chis_null[ix] = np.sum(data_cps ** 2 / t3err ** 2)
+        chis_bin[ix] = np.sum(cp_resids**2 / t3err**2)
+        chis_null[ix] = np.sum(data_cps**2 / t3err**2)
 
     # scale to red.chi2
     if dof == "Default":
@@ -2033,7 +1617,7 @@ def find_extra_error(
     print("Null  :", err_null)
 
     # Diagnostic plots
-    cp_err_plot = np.sqrt(cp_err ** 2 + err_bin ** 2)
+    cp_err_plot = np.sqrt(cp_err**2 + err_bin**2)
 
     plt.clf()
     plt.subplot(211)
@@ -2047,137 +1631,3 @@ def find_extra_error(
     plt.tight_layout()
 
     return chis_bin, extra_errors
-
-
-# =========================================================================
-# =========================================================================
-
-
-# def multiple_companions_nest(cpo, paramlimits, n_comp=2., resume=False, eff=0.3, multi=True,
-#                              err_scale=1., extra_error=0., plot=True, npoints=1000, logcrat_prior=True):
-#     '''Default implementation of a MultiNest fitting routine for closure
-#     phase data. Requires a closure phase cpo object, parameter limits and
-#     sensible keyword arguments for the multinest parameters.
-
-#     This function does very naughty things creating functions inside this
-#     function because PyMultiNest is very picky about how you pass it
-#     data.
-
-#     Optional parameter eff tunes sampling efficiency, and multi toggles multimodal
-#     nested sampling on and off. Turning off multimodal sampling results in a speed
-#     boost of ~ 20-30%.
-
-#     logcrat_prior allows the prior for the contrast ratios to be flat in logarithmic space
-
-#     paramlimits is needed here, and must be a list with length = ncomp,
-#     with paramlimits[companion_number]=[minsep,maxsep,minpa,maxpa,mincrat,maxcrat]
-#     '''
-#     import pymultinest
-
-#     if logcrat_prior:
-#         def myprior(cube, ndim, n_params, paramlimits=paramlimits, kpo=0, n_comp=n_comp):
-#             for comp_ix in range(0, n_comp):
-#                 cube[3*comp_ix+0] = cube[3*comp_ix+0] * \
-#                     (paramlimits[comp_ix][1] - paramlimits[comp_ix]
-#                      [0])+paramlimits[comp_ix][0]
-#                 cube[3*comp_ix+1] = cube[3*comp_ix+1] * \
-#                     (paramlimits[comp_ix][3] - paramlimits[comp_ix]
-#                      [2])+paramlimits[comp_ix][2]
-#                 cube[3*comp_ix+2] = 10**(cube[3*comp_ix+2]*(np.log10(paramlimits[comp_ix][5]) -
-#                                                             np.log10(paramlimits[comp_ix][4]))+np.log10(paramlimits[comp_ix][4]))
-
-#     else:
-#         def myprior(cube, ndim, n_params, paramlimits=paramlimits, kpo=0, n_comp=n_comp):
-#             n_comp = ndim/3
-#             for comp_ix in range(0, n_comp):
-#                 cube[3*comp_ix+0] = cube[3*comp_ix+0] * \
-#                     (paramlimits[comp_ix][1] - paramlimits[comp_ix]
-#                      [0])+paramlimits[comp_ix][0]
-#                 cube[3*comp_ix+1] = cube[3*comp_ix+1] * \
-#                     (paramlimits[comp_ix][3] - paramlimits[comp_ix]
-#                      [2])+paramlimits[comp_ix][2]
-#                 cube[3*comp_ix+2] = cube[3*comp_ix+2] * \
-#                     (paramlimits[comp_ix][5] - paramlimits[comp_ix]
-#                      [4])+paramlimits[comp_ix][4]
-
-#     def myloglike(cube, ndim, n_params):
-#         loglike = cp_loglikelihood_multiple(
-#             cube, cpo.u, cpo.v, cpo.wavel, cpo.t3data, cpo.t3err, ncomp=n_comp)
-#         return loglike
-
-#     # Add in the extra error and scale it
-#     cpo.t3err = np.sqrt(cpo.t3err**2+extra_error**2)
-#     cpo.t3err *= err_scale
-
-#     parameters = n_comp*['Separation', 'Position Angle', 'Contrast']
-#     n_params = 3*n_comp
-
-#     # Check that the "chains" directory exists (which multinest needs)
-#     if os.path.exists(os.getcwd()+'/chains/') is False:
-#         os.mkdir(os.getcwd()+'/chains/')
-
-#     tic = time.time()  # start timing
-
-#     # ---------------------------------
-#     # now run MultiNest!
-#     # ---------------------------------
-
-#     pymultinest.run(myloglike, myprior, n_comp*3, wrapped_params=[1], resume=resume, verbose=True,
-#                     sampling_efficiency=eff, multimodal=multi, n_iter_before_update=1000,
-#                     n_live_points=npoints)
-#     # let's analyse the results
-#     a = pymultinest.Analyzer(n_params=n_comp*3)
-#     s = a.get_stats()
-
-#     toc = time.time()
-
-#     if toc-tic < 60.:
-#         print('Time elapsed =', toc-tic, 's')
-#     else:
-#         print('Time elapsed =', (toc-tic)/60., 'mins')
-
-#     # json.dump(s, file('%s.json' % a.outputfiles_basename, 'w'), indent=2)
-#     print()
-#     print("-" * 30, 'ANALYSIS', "-" * 30)
-#     print("Global Evidence:\n\t%.15e +- %.15e" %
-#           (s['global evidence'], s['global evidence error']))
-#     params = s['marginals']
-
-#     for comp_ix in range(n_comp):
-#         bestsep = params[comp_ix*3+0]['median']
-#         seperr = params[comp_ix*3+0]['sigma']
-
-#         bestth = params[comp_ix*3+1]['median']
-#         therr = params[comp_ix*3+1]['sigma']
-
-#         bestcon = params[comp_ix*3+2]['median']
-#         conerr = params[comp_ix*3+2]['sigma']
-
-#         print('')
-#         print('Companion ', comp_ix, ':')
-#         print('Separation', bestsep, 'pm', seperr)
-#         print('Position angle', bestth, 'pm', therr)
-#         print('Contrast ratio', bestcon, 'pm', conerr)
-
-#     if plot:
-#         p = pymultinest.PlotMarginalModes(a)
-#         plt.figure(figsize=(5*n_params, 5*n_params))
-#         plt.clf()
-#         for i in range(n_params):
-#             plt.subplot(n_params, n_params, n_params * i + i + 1)
-#             p.plot_marginal(i, with_ellipses=True,
-#                             with_points=False, grid_points=20)
-#             plt.ylabel("Probability")
-#             plt.xlabel(parameters[i])
-
-#             for j in range(i):
-#                 plt.subplot(n_params, n_params, n_params * j + i + 1)
-#                 #plt.subplots_adjust(left=0, bottom=0, right=0, top=0, wspace=0, hspace=0)
-#                 p.plot_conditional(i, j, with_ellipses=False,
-#                                    with_points=True, grid_points=20)
-#                 plt.xlabel(parameters[i])
-#                 plt.ylabel(parameters[j])
-
-#     return [a, s]
-
-# =========================================================================
