@@ -1,16 +1,14 @@
 import glob
 import logging
 
-import matplotlib.pyplot as plt
 import numpy as np
-import scipy
-from scipy.io.idl import readsav
 from termcolor import cprint
 
-# import pymask.oifits
 from . import oifits
 from .cp_tools import project_cps
 
+# import pymask.oifits
+
 """------------------------------------------------------------------------
 cpo.py - Python class for manipulating oifits format closure phase data.
 ------------------------------------------------------------------------"""
@@ -25,16 +23,16 @@ class CustomFormatter(logging.Formatter):
     green = "\x1b[32;5m"
     bold_red = "\x1b[31;1m"
     reset = "\x1b[0m"
-    format = (
+    fmt = (
         "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
     )
 
     FORMATS = {
-        logging.DEBUG: grey + format + reset,
-        logging.INFO: green + format + reset,
-        logging.WARNING: yellow + format + reset,
-        logging.ERROR: red + format + reset,
-        logging.CRITICAL: bold_red + format + reset,
+        logging.DEBUG: grey + fmt + reset,
+        logging.INFO: green + fmt + reset,
+        logging.WARNING: yellow + fmt + reset,
+        logging.ERROR: red + fmt + reset,
+        logging.CRITICAL: bold_red + fmt + reset,
     }
 
     def format(self, record):
@@ -220,6 +218,8 @@ class icpo:
         """Import the closure phase data from the IDL pipeline directly
         from the bispectrum (bs) files"""
         # Get all of the bispectrum files
+        from scipy.io import readsav
+
         bs_files = glob.glob(analysis_dir + "bs*.idlvar")
         bs_files.sort()
 
@@ -425,6 +425,8 @@ class icpo:
             tol: the minimum value used to detect significant eigenvalues. This is
                     ignored if n_significant is set
         """
+        import scipy
+
         proj = []
 
         for wav_ix, wav in enumerate(self.wavel):
@@ -453,6 +455,8 @@ class icpo:
                     )
 
             if plot:
+                import matplotlib.pyplot as plt
+
                 plt.clf()
                 plt.plot(evals)
                 plt.xlabel("Mode number")
@@ -529,6 +533,8 @@ class PolDiffObs:
         the files output by the IDL masking pipeline."""
 
         # Use scipy's idl reading function to get a dictionary of all the saved variables
+        from scipy.io import readsav
+
         data = readsav(filename)
 
         # The differential visibility quantities
