@@ -69,18 +69,14 @@ list of all the "hidden" attributes.
 For further information, contact Paul Boley (pboley@urfu.ru).
 
 """
-
 import copy
 import datetime
 import warnings
 
 import numpy as np
-from numpy import double, ma
+from numpy import double
+from numpy import ma
 
-try:
-    import pyfits
-except ImportError:
-    from astropy.io import fits as pyfits
 
 __author__ = "Paul Boley"
 __email__ = "pboley@gmail.com"
@@ -394,7 +390,7 @@ class OI_VIS:
                 len(self.visamp),
                 _plurals(len(self.visamp)),
                 np.sum(self.flag),
-                np.sqrt(self.ucoord ** 2 + self.vcoord ** 2),
+                np.sqrt(self.ucoord**2 + self.vcoord**2),
                 np.arctan(self.ucoord / self.vcoord) * 180.0 / np.pi % 180.0,
                 meanvis,
             )
@@ -482,16 +478,19 @@ class OI_VIS2:
             )
         else:
             baselinename = ""
-        return "%s %s%s: %d point%s (%d masked), B = %5.1f m, PA = %5.1f deg, <V^2> = %4.2g" % (
-            self.target.target,
-            self.timeobs.strftime("%F %T"),
-            baselinename,
-            len(self.vis2data),
-            _plurals(len(self.vis2data)),
-            np.sum(self.flag),
-            np.sqrt(self.ucoord ** 2 + self.vcoord ** 2),
-            np.arctan(self.ucoord / self.vcoord) * 180.0 / np.pi % 180.0,
-            meanvis,
+        return (
+            "%s %s%s: %d point%s (%d masked), B = %5.1f m, PA = %5.1f deg, <V^2> = %4.2g"
+            % (
+                self.target.target,
+                self.timeobs.strftime("%F %T"),
+                baselinename,
+                len(self.vis2data),
+                _plurals(len(self.vis2data)),
+                np.sum(self.flag),
+                np.sqrt(self.ucoord**2 + self.vcoord**2),
+                np.arctan(self.ucoord / self.vcoord) * 180.0 / np.pi % 180.0,
+                meanvis,
+            )
         )
 
     def info(self):
@@ -599,8 +598,8 @@ class OI_T3:
             len(self.t3amp),
             _plurals(len(self.t3amp)),
             np.sum(self.flag),
-            np.sqrt(self.u1coord ** 2 + self.v1coord ** 2),
-            np.sqrt(self.u2coord ** 2 + self.v2coord ** 2),
+            np.sqrt(self.u1coord**2 + self.v1coord**2),
+            np.sqrt(self.u2coord**2 + self.v2coord**2),
             meant3,
         )
 
@@ -689,7 +688,7 @@ class OI_ARRAY:
 
     def __getattr__(self, attrname):
         if attrname == "latitude":
-            radius = np.sqrt((self.arrxyz ** 2).sum())
+            radius = np.sqrt((self.arrxyz**2).sum())
             if radius == 0.0:
                 warnings.warn(
                     "Warning: ARRAYX, ARRAYY, ARRAYZ are all zero", UserWarning
@@ -697,7 +696,7 @@ class OI_ARRAY:
                 return _angpoint(np.nan)
             return _angpoint(np.arcsin(self.arrxyz[2] / radius) * 180.0 / np.pi)
         elif attrname == "longitude":
-            radius = np.sqrt((self.arrxyz ** 2).sum())
+            radius = np.sqrt((self.arrxyz**2).sum())
             if radius == 0.0:
                 warnings.warn(
                     "Warning: ARRAYX, ARRAYY, ARRAYZ are all zero", UserWarning
@@ -706,7 +705,7 @@ class OI_ARRAY:
             xylen = np.sqrt(self.arrxyz[0] ** 2 + self.arrxyz[1] ** 2)
             return _angpoint(np.arcsin(self.arrxyz[1] / xylen) * 180.0 / np.pi)
         elif attrname == "altitude":
-            radius = np.sqrt((self.arrxyz ** 2).sum())
+            radius = np.sqrt((self.arrxyz**2).sum())
             if radius == 0.0:
                 warnings.warn(
                     "Warning: ARRAYX, ARRAYY, ARRAYZ are all zero", UserWarning
@@ -1209,6 +1208,7 @@ class oifits:
     def save(self, filename):
         """Write the contents of the oifits object to a file in OIFITS
         format."""
+        from astropy.io import fits as pyfits
 
         if not self.isconsistent():
             raise ValueError("oifits object is not consistent; refusing to go further")
@@ -1890,8 +1890,32 @@ class oifits:
         hdulist.writeto(filename, clobber=True)
 
 
+def get_timeobs(header, row):
+    """
+    Get observation time based on information from oifits header and data row.
+    If no DATE-OBS is found in the header, only the TIME from `row` is used.
+    """
+    if "T" in header["DATE-OBS"]:
+        warnings.warn(
+            "Warning: DATE-OBS contains a timestamp, which is contradictory to the OIFITS2 standard",
+            UserWarning,
+        )
+        date = header["DATE-OBS"].split("T")[0].split("-")
+    else:
+        date = header["DATE-OBS"].split("-")
+    if date[0] != "":
+        timeobs = datetime.datetime(
+            int(date[0]), int(date[1]), int(date[2])
+        ) + datetime.timedelta(seconds=np.around(row.field("TIME"), 2))
+    else:
+        timeobs = datetime.timedelta(seconds=np.around(row.field("TIME"), 2))
+
+    return timeobs
+
+
 def open(filename, quiet=True):
     """Open an OIFITS file."""
+    from astropy.io import fits
 
     newobj = oifits()
     targetmap = {}
@@ -1899,7 +1923,7 @@ def open(filename, quiet=True):
 
     if not quiet:
         print("Opening %s" % filename)
-    hdulist = pyfits.open(filename)
+    hdulist = fits.open(filename)
     # Save the primary header
     newobj.header = hdulist[0].header.copy()
 
@@ -1910,7 +1934,7 @@ def open(filename, quiet=True):
         # PyFITS 2.4 had a bug where strings in binary tables were padded with
         # spaces instead of nulls.  This was fixed in PyFITS 3.0.0, but many files
         # suffer from this problem, and the strings are ugly as a result.  Fix it.
-        if type(hdu) == pyfits.hdu.table.BinTableHDU:
+        if type(hdu) == fits.hdu.table.BinTableHDU:
             for name in data.names:
                 if data.dtype[name].type == np.string_:
                     data[name] = list(map(str.rstrip, data[name]))
@@ -1961,17 +1985,7 @@ def open(filename, quiet=True):
             wavelength = newobj.wavelength[header["INSNAME"]]
         if hdu.name == "OI_VIS":
             for row in data:
-                if "T" in header["DATE-OBS"]:
-                    warnings.warn(
-                        "Warning: DATE-OBS contains a timestamp, which is contradictory to the OIFITS2 standard",
-                        UserWarning,
-                    )
-                    date = header["DATE-OBS"].split("T")[0].split("-")
-                else:
-                    date = header["DATE-OBS"].split("-")
-                timeobs = datetime.datetime(
-                    int(date[0]), int(date[1]), int(date[2])
-                ) + datetime.timedelta(seconds=np.around(row.field("TIME"), 2))
+                timeobs = get_timeobs(header, row)
                 int_time = row.field("INT_TIME")
                 visamp = np.reshape(row.field("VISAMP"), -1)
                 visamperr = np.reshape(row.field("VISAMPERR"), -1)
@@ -2018,17 +2032,7 @@ def open(filename, quiet=True):
                 )
         elif hdu.name == "OI_VIS2":
             for row in data:
-                if "T" in header["DATE-OBS"]:
-                    warnings.warn(
-                        "Warning: DATE-OBS contains a timestamp, which is contradictory to the OIFITS2 standard",
-                        UserWarning,
-                    )
-                    date = header["DATE-OBS"].split("T")[0].split("-")
-                else:
-                    date = header["DATE-OBS"].split("-")
-                timeobs = datetime.datetime(
-                    int(date[0]), int(date[1]), int(date[2])
-                ) + datetime.timedelta(seconds=np.around(row.field("TIME"), 2))
+                timeobs = get_timeobs(header, row)
                 int_time = row.field("INT_TIME")
                 vis2data = np.reshape(row.field("VIS2DATA"), -1)
                 vis2err = np.reshape(row.field("VIS2ERR"), -1)
@@ -2061,17 +2065,7 @@ def open(filename, quiet=True):
                 )
         elif hdu.name == "OI_T3":
             for row in data:
-                if "T" in header["DATE-OBS"]:
-                    warnings.warn(
-                        "Warning: DATE-OBS contains a timestamp, which is contradictory to the OIFITS2 standard",
-                        UserWarning,
-                    )
-                    date = header["DATE-OBS"].split("T")[0].split("-")
-                else:
-                    date = header["DATE-OBS"].split("-")
-                timeobs = datetime.datetime(
-                    int(date[0]), int(date[1]), int(date[2])
-                ) + datetime.timedelta(seconds=np.around(row.field("TIME"), 2))
+                timeobs = get_timeobs(header, row)
                 int_time = row.field("INT_TIME")
                 t3amp = np.reshape(row.field("T3AMP"), -1)
                 t3amperr = np.reshape(row.field("T3AMPERR"), -1)
