@@ -13,19 +13,27 @@ All AMI related function, the most important are:
 
 --------------------------------------------------------------------
 """
+import os
+from pathlib import Path
 
 import numpy as np
-from matplotlib import pyplot as plt
-from munch import munchify as dict2class
 from termcolor import cprint
 
 from amical.dpfit import leastsqFit
-from amical.get_infos_obs import get_mask, get_pixel_size, get_wavelength
-from amical.mf_pipeline.idl_function import array_coords, dist
-from amical.tools import gauss_2d_asym, linear, norm_max, plot_circle
+from amical.get_infos_obs import get_mask
+from amical.get_infos_obs import get_pixel_size
+from amical.get_infos_obs import get_wavelength
+from amical.mf_pipeline.idl_function import array_coords
+from amical.mf_pipeline.idl_function import dist
+from amical.tools import gauss_2d_asym
+from amical.tools import linear
+from amical.tools import norm_max
+from amical.tools import plot_circle
 
 
 def _plot_mask_coord(xy_coords, maskname, instrument):
+    import matplotlib.pyplot as plt
+
     if instrument == "NIRISS":
         marker = "H"
         D = 6.5
@@ -51,7 +59,6 @@ def _plot_mask_coord(xy_coords, maskname, instrument):
     plt.ylabel("Aperture y-coordinate [m]", fontsize=12)
     plt.axis([-D / 2.0, D / 2.0, -D / 2.0, D / 2.0])
     plt.tight_layout()
-    plt.show(block=False)
     return fig
 
 
@@ -120,7 +127,7 @@ def _peak_fft_method(
         ap2[x2 + 1, y2] = (1.0 - delta[0]) * delta[1]
         ap2[x2 + 1, y2 + 1] = delta[0] * delta[1]
 
-        n_elts = npix ** 2
+        n_elts = npix**2
 
         tmf = np.fft.fft2(ap1) / n_elts * np.conj(np.fft.fft2(ap2) / n_elts)
         tmf = np.fft.fft2(tmf)
@@ -183,10 +190,12 @@ def _peak_gauss_method(
     fw_splodge=0.7,
     hole_diam=0.8,
 ):
+    from munch import munchify as dict2class
+
     mf = np.zeros([npix, npix])
     n_holes = index_mask.n_holes
 
-    l_B = np.sqrt(u ** 2 + v ** 2)
+    l_B = np.sqrt(u**2 + v**2)
     minbl = np.min(l_B) * filt[0]
     if n_holes >= 15:
         sampledisk_r = minbl / 2 / filt[0] * pixelsize * npix * 0.9
@@ -296,6 +305,8 @@ def _make_overlap_mat(mf, n_baselines, display=False):
     mf_imat[mf_imat <= -2] = -2
 
     if display:
+        import matplotlib.pyplot as plt
+
         plt.figure(figsize=(6, 6))
         plt.title("Overlap matrix", fontsize=14)
         plt.imshow(mf_imat, cmap="gray", origin="upper")
@@ -322,6 +333,8 @@ def make_mf(
     diag_plot=False,
     verbose=False,
     display=True,
+    save_to=None,
+    filename=None,
 ):
     """
     Summary:
@@ -360,6 +373,7 @@ def make_mf(
         Relative size of the splodge used to compute multiple triangle indices and the fwhm
         of the 'gauss' technique,\n
     """
+    from munch import munchify as dict2class
 
     # Get detector, filter and mask informations
     # ------------------------------------------
@@ -371,6 +385,12 @@ def make_mf(
     filt = get_wavelength(instrument, filtname)
 
     if instrument == "SPHERE-IFS":
+        if i_wl is None:
+            raise ValueError(
+                "Your file seems to be obtained with IFU instrument: spectral "
+                f"channel index `i_wl` must be specified (nlambda = {len(filt)}) and "
+                "should be strictly identical to the one used for the cleaning step."
+            )
         if isinstance(i_wl, (int, np.integer)):
             filt = [filt[i_wl], 0.001 * filt[i_wl]]
         else:
@@ -394,7 +414,12 @@ def make_mf(
     xy_coords = np.array(xy_coords_rot)
 
     if display:
+        import matplotlib.pyplot as plt
+
         _plot_mask_coord(xy_coords, maskname, instrument)
+        if save_to is not None:
+            figname = os.path.join(save_to, Path(filename).stem)
+            plt.savefig(f"{figname}_{1}.pdf")
 
     n_holes = xy_coords.shape[0]
 
@@ -549,7 +574,9 @@ def make_mf(
     im_uv = np.roll(np.fft.fftshift(mf_tot), 1, axis=1)
 
     if display:
-        plt.figure(figsize=(6, 6))
+        import matplotlib.pyplot as plt
+
+        plt.figure(figsize=(9, 7))
         plt.title("(u-v) plan - mask %s" % (maskname), fontsize=14)
         plt.imshow(im_uv, origin="lower")
         plt.plot(npix // 2 + 1, npix // 2, "r+")
@@ -612,6 +639,7 @@ def compute_index_mask(n_holes, verbose=False):
         Bispectrum covariance to bispectrum index.
 
     """
+    from munch import munchify as dict2class
 
     n_baselines = int(n_holes * (n_holes - 1) / 2)
     n_bispect = int(n_holes * (n_holes - 1) * (n_holes - 2) / 6)
@@ -828,6 +856,8 @@ def tri_pix(array_size, sampledisk_r, verbose=True, display=True):
     fw = 2 * sampledisk_r
 
     if display:
+        import matplotlib.pyplot as plt
+
         plt.figure(figsize=(5, 5))
         plt.title(
             "Splodge + unique triangle (tri = %i/%i, d = %i, r = %2.1f pix)"
@@ -951,17 +981,13 @@ def bs_multi_triangle(i, bs_arr, ft_frame, bs2bl_ix, mf, closing_tri_pix):
     return bs_arr
 
 
-def find_bad_holes(res_c, n_holes, bl2h_ix, bmax=6, verbose=False, display=False):
+def find_bad_holes(bs, bmax=6, verbose=False, display=False):
     """Find bad apertures using a linear fit of the v2 vs. spatial frequencies.
 
     Parameters
     ----------
-    `res_c` : {class}
-        Class containing NRM data of the calibrator (bispect.py),\n
-    `n_holes` : {int}
-        Number of apertures,\n
-    `bl2h_ix` : {array}
-        Baselines to apertures (holes) indices,\n
+    `bs` : {class}
+        Class containing NRM data (bispect.py),\n
     `bmax` : {int}, optional
         Maximum baseline used to plot the fit, by default 6,\n
     `verbose` : {bool}, optional
@@ -974,21 +1000,26 @@ def find_bad_holes(res_c, n_holes, bl2h_ix, bmax=6, verbose=False, display=False
     `bad_holes`: {array}
         List of determined bad holes,
     """
-    u = res_c.u / res_c.wl
-    v = res_c.v / res_c.wl
-    X = np.sqrt(u ** 2 + v ** 2)
-    Y = np.log(res_c.v2)
+    u = bs.u / bs.wl
+    v = bs.v / bs.wl
+    X = np.sqrt(u**2 + v**2)
+    Y = np.log(bs.vis2)
+
+    n_holes = bs.mask.n_holes
+    bl2h_ix = bs.mask.bl2h_ix
 
     param = {"a": 1, "b": 0}
 
     fit = leastsqFit(linear, X, param, Y, verbose=verbose)
 
-    xm = np.linspace(0, bmax, 100) / res_c.wl
+    xm = np.linspace(0, bmax, 100) / bs.wl
     ym = linear(xm, fit["best"])
 
     pfit = list(fit["best"].values())
 
     if display:
+        import matplotlib.pyplot as plt
+
         plt.figure()
         plt.plot(X / 1e6, Y, ".", label="data")
         plt.plot(xm / 1e6, ym, "--", label=f"fit (a={pfit[0]:2.1e}, b={pfit[1]:2.2f})")
@@ -1001,9 +1032,8 @@ def find_bad_holes(res_c, n_holes, bl2h_ix, bmax=6, verbose=False, display=False
 
     bad_holes = []
     for j in range(n_holes):
-        pass
         w = np.where((bl2h_ix[0, :] == j) | (bl2h_ix[1, :] == j))
-        if (np.mean(res_c.v2[w] / np.exp(fit["model"][w]))) <= 0.3:
+        if (np.mean(bs.vis2[w] / np.exp(fit["model"][w]))) <= 0.3:
             bad_holes.append(j)
 
     bad_holes = np.unique(bad_holes)
@@ -1011,7 +1041,7 @@ def find_bad_holes(res_c, n_holes, bl2h_ix, bmax=6, verbose=False, display=False
     return bad_holes
 
 
-def find_bad_BL_BS(bad_holes, bl2h_ix, bs2bl_ix):
+def find_bad_BL_BS(bad_holes, bs):
     """
     Give indices of bad BS and V2 using a given bad holes list.
 
@@ -1035,6 +1065,10 @@ def find_bad_BL_BS(bad_holes, bl2h_ix, bs2bl_ix):
         Good bispectrum indices.
 
     """
+
+    bl2h_ix = bs.mask.bl2h_ix
+    bs2bl_ix = bs.mask.bs2bl_ix
+
     n_baselines = bl2h_ix.shape[1]
     n_bispect = bs2bl_ix.shape[1]
 
