@@ -12,34 +12,42 @@ and calc_bispect.pro).
 
 --------------------------------------------------------------------
 """
-
+import os
 import sys
 import time
 import warnings
+from pathlib import Path
 
 import numpy as np
-from astropy.io import fits
-from matplotlib import pyplot as plt
-from mpl_toolkits.axes_grid1 import make_axes_locatable
-from munch import munchify as dict2class
-from scipy.optimize import minimize
 from termcolor import cprint
 from tqdm import tqdm
 
+from .idl_function import dblarr
+from .idl_function import dist
+from .idl_function import regress_noc
 from amical.get_infos_obs import get_mask
-from amical.mf_pipeline.ami_function import (
-    bs_multi_triangle,
-    compute_index_mask,
-    give_peak_info2d,
-    make_mf,
-    phase_chi2,
-    tri_pix,
-)
-from amical.tools import compute_pa, cov2cor
+from amical.mf_pipeline.ami_function import bs_multi_triangle
+from amical.mf_pipeline.ami_function import compute_index_mask
+from amical.mf_pipeline.ami_function import give_peak_info2d
+from amical.mf_pipeline.ami_function import make_mf
+from amical.mf_pipeline.ami_function import phase_chi2
+from amical.mf_pipeline.ami_function import tri_pix
+from amical.tools import compute_pa
+from amical.tools import cov2cor
 
-from .idl_function import dblarr, dist, regress_noc
 
-warnings.filterwarnings("ignore")
+def _PYPDF2_VERSION():
+    from importlib.metadata import version
+    from packaging.version import Version
+
+    return Version(version("PyPDF2"))
+
+
+def _ASTROPY_VERSION():
+    from importlib.metadata import version
+    from packaging.version import Version
+
+    return Version(version("astropy"))
 
 
 def _compute_complex_bs(
@@ -50,6 +58,7 @@ def _compute_complex_bs(
     dark_ps=None,
     closing_tri_pix=None,
     bs_multi_tri=False,
+    verbose=True,
 ):
     """Compute the complex visibilities and the bispectrum of an input ft_arr (_construct_ft_arr()).
     In addition, compute the phase phs of each frames, and some calibration array (relative to the
@@ -116,6 +125,7 @@ def _compute_complex_bs(
         desc="Extracting in the cube",
         leave=False,
         file=sys.stdout,
+        disable=np.invert(verbose),
     ):
         ft_frame = ft_arr[i]
         ps = np.abs(ft_frame) ** 2
@@ -139,7 +149,7 @@ def _compute_complex_bs(
             pix = fringe_peak[j][:, 0].astype(int), fringe_peak[j][:, 1].astype(int)
             gain = fringe_peak[j][:, 2]
 
-            calib_v2["dark"][j] = np.sum(gain ** 2 * dps[pix])
+            calib_v2["dark"][j] = np.sum(gain**2 * dps[pix])
             cvis[j] = np.sum(gain * ft_frame[pix])
 
             ftf1 = np.roll(ft_frame, 1, axis=0)
@@ -238,6 +248,8 @@ def _show_complex_ps(ft_arr, i_frame=0):
     Show the complex fft image (real and imaginary) and power spectrum (abs(fft)) of the first frame
     to check the applied correction on the cube.
     """
+    import matplotlib.pyplot as plt
+
     fig = plt.figure(figsize=(16, 6))
     ax1 = plt.subplot(1, 3, 1)
     plt.title("Real part")
@@ -257,6 +269,9 @@ def _show_peak_position(
 ):
     """Show the expected position of the peak in the Fourier space using the
     mask coordinates and the chosen method."""
+    import matplotlib.pyplot as plt
+    from mpl_toolkits.axes_grid1 import make_axes_locatable
+
     dim1, dim2 = ft_arr.shape[1], ft_arr.shape[2]
     x, y = np.arange(dim1), np.arange(dim2)
     X, Y = np.meshgrid(x, y)
@@ -311,6 +326,7 @@ def _show_peak_position(
 def _show_norm_matrices(obs_norm, expert_plot=False):
     """Show covariances matrices of the V2, CP, and a combination
     bispectrum vs. V2."""
+    import matplotlib.pyplot as plt
 
     v2_cov = obs_norm["v2_cov"]
     cp_cov = obs_norm["cp_cov"]
@@ -348,6 +364,8 @@ def _check_input_infos(hdr, targetname=None, filtname=None, instrum=None, verbos
     input arguments. Return the infos class containing important informations of
     the input header (keys: target, seeing, instrument, ...)
     """
+    from munch import munchify as dict2class
+
     target = hdr.get("OBJECT")
     filt = hdr.get("FILTER")
     instrument = hdr.get("INSTRUME", instrum)
@@ -367,7 +385,7 @@ def _check_input_infos(hdr, targetname=None, filtname=None, instrum=None, verbos
                     "green",
                 )
         else:
-            cprint("Error: target name not found (header or as input).", "red")
+            cprint("Warning: target name not found (header or as input).", "green")
 
     # Check the filter used
     if filt is None:
@@ -379,12 +397,10 @@ def _check_input_infos(hdr, targetname=None, filtname=None, instrum=None, verbos
                     % filtname,
                     "green",
                 )
-        else:
-            cprint("Error: filter not found (in the header or as input).", "red")
 
     # Check the instrument used
     if instrument is None:
-        cprint("Error: instrum not found (in the header or as input).", "red")
+        raise OSError("instrum not found (in the header or as input).")
 
     # Origin files
     orig = hdr.get("ORIGFILE", "SimulatedData")
@@ -501,7 +517,7 @@ def _unbias_v2_arr(
 
         autocor_mf = np.fft.ifft2(np.abs(np.fft.ifft2(im_peak)) ** 2).real
         bias_arr[j] = (
-            np.sum(autocor_mf * autocor_noise) * bias / autocor_noise[0, 0] * npix ** 2
+            np.sum(autocor_mf * autocor_noise) * bias / autocor_noise[0, 0] * npix**2
         )
 
         if unbias:
@@ -548,10 +564,10 @@ def _compute_v2_quantities(v2_arr, bias_arr, n_blocks):
     # v2_cov = v2_cov_pyt
 
     x = np.arange(n_baselines)
-    avar = v2_cov[x, x] * n_ps - bias_arr ** 2 * (1 + (2.0 * v2) / bias_arr)
+    avar = v2_cov[x, x] * n_ps - bias_arr**2 * (1 + (2.0 * v2) / bias_arr)
     err_avar = np.sqrt(
-        2.0 / n_ps * (v2_cov[x, x]) ** 2 * n_ps ** 2
-        + 4.0 * v2_cov[x, x] * bias_arr ** 2
+        2.0 / n_ps * (v2_cov[x, x]) ** 2 * n_ps**2
+        + 4.0 * v2_cov[x, x] * bias_arr**2
     )
 
     v2_quantities = {
@@ -642,13 +658,15 @@ def _compute_bs_cov(bs_arr, bs, bscov2bs_ix, n_cov):
     return bs_cov
 
 
-def _compute_cp_cov(bs_arr, bs, index_mask):
+def _compute_cp_cov(bs_arr, bs, index_mask, disable=False):
     """Compute the covariance matrix of the closure phase."""
     n_ps = bs_arr.shape[0]
     n_bispect = index_mask.n_bispect
 
     cp_cov = dblarr(n_bispect, n_bispect)
-    for i in tqdm(range(n_bispect), desc="CP covariance", ncols=100, leave=False):
+    for i in tqdm(
+        range(n_bispect), desc="CP covariance", ncols=100, leave=False, disable=disable
+    ):
         for j in range(n_bispect):
             temp1 = (bs_arr[:, i] - bs[i]) * np.conj(bs[i])
             temp2 = (bs_arr[:, j] - bs[j]) * np.conj(bs[j])
@@ -690,6 +708,7 @@ def _normalize_all_obs(
     index_mask,
     infos,
     expert_plot=False,
+    save=False,
 ):
     """Normalize all observables by the appropriate factor proportional to
     the averaged fluxes and the number of holes."""
@@ -707,29 +726,31 @@ def _normalize_all_obs(
 
     n_holes = index_mask.n_holes
 
-    bs_arr_norm = bs_arr / np.mean(fluxes ** 3) * n_holes ** 3
-    v2_arr_norm = v2_arr / np.mean(fluxes ** 2) * n_holes ** 2
+    bs_arr_norm = bs_arr / np.mean(fluxes**3) * n_holes**3
+    v2_arr_norm = v2_arr / np.mean(fluxes**2) * n_holes**2
     cvis_arr_norm = cvis_arr / np.mean(fluxes) * n_holes
 
-    v2_norm = (v2 / np.mean(fluxes ** 2)) * n_holes ** 2
-    v2_cov_norm = (v2_cov / np.mean(fluxes ** 4)) * n_holes ** 4
+    v2_norm = (v2 / np.mean(fluxes**2)) * n_holes**2
+    v2_cov_norm = (v2_cov / np.mean(fluxes**4)) * n_holes**4
 
-    bs_norm = bs / np.mean(fluxes ** 3) * n_holes ** 3
+    bs_norm = bs / np.mean(fluxes**3) * n_holes**3
 
-    avar_norm = avar / np.mean(fluxes ** 4) * n_holes ** 4
-    err_avar_norm = err_avar / np.mean(fluxes ** 4) * n_holes ** 4
+    avar_norm = avar / np.mean(fluxes**4) * n_holes**4
+    err_avar_norm = err_avar / np.mean(fluxes**4) * n_holes**4
 
     try:
-        cp_cov_norm = cp_cov / np.mean(fluxes ** 6) * n_holes ** 6
+        cp_cov_norm = cp_cov / np.mean(fluxes**6) * n_holes**6
     except TypeError:
         cp_cov_norm = None
 
-    bs_cov_norm = bs_cov / np.mean(fluxes ** 6) * n_holes ** 6
-    bs_v2_cov_norm = np.real(bs_v2_cov / np.mean(fluxes ** 5) * n_holes ** 5)
-    bs_var_norm = bs_var / np.mean(fluxes ** 6) * n_holes ** 6
+    bs_cov_norm = bs_cov / np.mean(fluxes**6) * n_holes**6
+    bs_v2_cov_norm = np.real(bs_v2_cov / np.mean(fluxes**5) * n_holes**5)
+    bs_var_norm = bs_var / np.mean(fluxes**6) * n_holes**6
 
     if expert_plot:
-        plt.figure(figsize=(5, 4))
+        import matplotlib.pyplot as plt
+
+        plt.figure(figsize=(12, 6))
         plt.title("DIAGNOSTIC PLOTS - V2 - %s" % infos.target)
         plt.plot(v2_arr_norm[0], color="grey", alpha=0.2, label="V$^2$ dispersion")
         plt.plot(v2_arr_norm.T, color="grey", alpha=0.2)
@@ -774,7 +795,9 @@ def _compute_cp(obs_result, obs_norm, infos, expert_plot=False):
     obs_norm["cp_arr"] = cp_arr
 
     if expert_plot:
-        plt.figure(figsize=(5, 4))
+        import matplotlib.pyplot as plt
+
+        plt.figure(figsize=(12, 6))
         plt.title("DIAGNOSTIC PLOTS - CP - %s" % infos.target)
         plt.plot(cp_arr[0], color="grey", alpha=0.2, label="CP dispersion")
         plt.plot(cp_arr.T, color="grey", alpha=0.2)
@@ -840,6 +863,8 @@ def _compute_phs_piston(
 ):
     """Compute the phase piston to determine the additional phase error due to
     the wavefront differences between holes."""
+    from scipy.optimize import minimize
+
     n_holes = index_mask.n_holes
     n_baselines = index_mask.n_baselines
 
@@ -891,6 +916,8 @@ def _compute_phs_piston(
             pass
 
     if display:
+        import matplotlib.pyplot as plt
+
         plt.figure()
         plt.errorbar(
             np.arange(len(ph_mn)),
@@ -960,7 +987,7 @@ def _compute_phs_error(complex_bs, fitmat, index_mask, npix, imsize=3):
 
         tmp1 = hole_phs[0, j, bl2h_ix[0, :]] - hole_phs[0, j, bl2h_ix[1, :]]
         tmp2 = hole_phs[1, j, bl2h_ix[0, :]] - hole_phs[1, j, bl2h_ix[1, :]]
-        err[j, :] = tmp1 ** 2 + tmp2 ** 2
+        err[j, :] = tmp1**2 + tmp2**2
         err_bias[j, :] = (
             hole_err_phs[0, j, bl2h_ix[0, :]] - hole_err_phs[0, j, bl2h_ix[1, :]]
         ) ** 2 + (
@@ -976,32 +1003,47 @@ def _compute_phs_error(complex_bs, fitmat, index_mask, npix, imsize=3):
     # on real data (NB there is no window size adjustment here).
     phs_v2corr = np.zeros(n_baselines)
     for j in range(n_baselines):
-        phs_v2corr[j] = np.mean(np.exp(-2.5 * predictor[:, j] / imsize ** 2))
+        phs_v2corr[j] = np.mean(np.exp(-2.5 * predictor[:, j] / imsize**2))
 
     return phs_v2corr
 
 
 def _add_infos_header(infos, hdr, mf, pa, filename, maskname, npix):
     """Save important informations and some parts of the original header."""
+    from astropy.io import fits
+    from packaging.version import Version
+
     infos["pixscale"] = mf.pixelSize
     infos["pa"] = pa
     infos["filename"] = filename
     infos["maskname"] = maskname
     infos["isz"] = npix
 
-    # HACK: astropy _HeaderCommentaryCards are registered as mappings,
-    # so munch tries to access their keys, leading to attribute error
-    # to prevent this, we remove commentary cards as a temporary fix.
-    # (As of June 23 2021, with astropy version 4.2.1)
-    # See:
-    # https://github.com/SydneyAstrophotonicInstrumentationLab/AMICAL/issues/31
-    # https://github.com/astropy/astropy/issues/11866
+    # Raise a warning that old astropy version drop commentary cards, but only if
+    # there are any in the original header
     hdr_commentary_keys = fits.Card._commentary_keywords
-    hdr = hdr.copy()
-    for key in hdr_commentary_keys:
-        hdr.remove(key, ignore_missing=True, remove_all=True)
+    if any(hck in hdr for hck in hdr_commentary_keys) and (
+        _ASTROPY_VERSION() < Version("5.0rc")
+    ):
+        warnings.warn(
+            "Commentary cards are removed from the header with astropy"
+            f" version < 5.0. Your astropy version is"
+            f" {_ASTROPY_VERSION()}",
+            RuntimeWarning,
+        )
+        # HACK: astropy _HeaderCommentaryCards are registered as mappings,
+        # so munch tries to access their keys, leading to attribute error
+        # to prevent this, we remove commentary cards as a temporary fix.
+        # (As of September 2 2021, with astropy version 4.3.1)
+        # See:
+        # https://github.com/SydneyAstrophotonicInstrumentationLab/AMICAL/issues/31
+        # https://github.com/astropy/astropy/issues/11866
+        # Resolved upstream with
+        # https://github.com/astropy/astropy/pull/11923
+        hdr = hdr.copy()
+        for key in hdr_commentary_keys:
+            hdr.remove(key, ignore_missing=True, remove_all=True)
 
-    # Now that header is compatible with munch, we add it to infos
     infos["hdr"] = hdr
 
     # Save keys of the original header (as needed):
@@ -1009,9 +1051,39 @@ def _add_infos_header(infos, hdr, mf, pa, filename, maskname, npix):
     if infos.orig != "SimulatedData":
         for keys in add_keys:
             infos[keys.lower()] = hdr.get(keys)
+    else:
+        # For simulated data, add keys only if they exist
+        # (missing keys will be filled later if needed)
+        for keys in add_keys:
+            try:
+                infos[keys.lower()] = hdr[keys]
+            except KeyError:
+                pass
     return infos
 
 
+def produce_result_pdf(figdir, filename):
+    from packaging.version import Version
+
+    if _PYPDF2_VERSION() >= Version("1.28"):
+        from PyPDF2 import PdfMerger
+        from PyPDF2 import PdfReader
+    else:
+        from PyPDF2 import PdfFileMerger as PdfMerger
+        from PyPDF2 import PdfFileReader as PdfReader
+    # Call the PdfMerger
+    mergedObject = PdfMerger()
+
+    for fileNumber in range(7):
+        ifile = os.path.join(figdir, f"{filename}_{fileNumber + 1}.pdf")
+        mergedObject.append(PdfReader(ifile, "rb"))
+        os.remove(ifile)
+
+    # Write all the files into a file which is named as shown below
+    mergedObject.write(os.path.join(figdir, filename + "_DIAGNOSTIC_PLOTS.pdf"))
+    return 0
+
+
 def extract_bs(
     cube,
     filename,
@@ -1033,6 +1105,7 @@ def extract_bs(
     unbias_v2=True,
     compute_cp_cov=True,
     expert_plot=False,
+    save_to=None,
     verbose=False,
     display=True,
 ):
@@ -1084,6 +1157,8 @@ def extract_bs(
     `targetname` {str}:
         Name of the target to save in oifits file (if not in header of the
         cube),\n
+    `save_to` {str}:
+        Name of the repository to save the figures,\n
     `verbose` {bool}:
         If True, print usefull informations during the process.\n
     `display` {bool}:
@@ -1097,17 +1172,35 @@ def extract_bs(
         and the important information (.infos). The .mask, .infos and .matrix are also class with
         various quantities (see .mask.__dict__.keys()).
     """
+    from munch import munchify as dict2class
+    from astropy.io import fits
+
     if verbose:
         cprint("\n-- Starting extraction of observables --", "cyan")
     start_time = time.time()
 
-    hdu = fits.open(filename)
-    hdr = hdu[0].header
+    if save_to is not None:
+        if not os.path.exists(save_to):
+            os.mkdir(save_to)
+
+    with fits.open(filename) as hdu:
+        hdr = hdu[0].header
+        try:
+            sci_hdr = hdu["SCI"].header
+        except KeyError:
+            sci_hdr = None
 
     infos = _check_input_infos(
         hdr, targetname=targetname, filtname=filtname, instrum=instrum, verbose=False
     )
 
+    if infos.instrument == "SPHERE-IFS":
+        if i_wl is None:
+            raise ValueError(
+                "Your file seems to be obtained with an IFU instrument: spectral "
+                "channel index `i_wl` must be specified."
+            )
+
     if "INSTRUME" not in hdr.keys():
         hdr["INSTRUME"] = infos["instrument"]
     # 1. Open the data cube and perform a series of roll (both axis) to avoid
@@ -1147,7 +1240,19 @@ def extract_bs(
         theta_detector=theta_detector,
         i_wl=i_wl,
         display=display,
+        save_to=save_to,
+        filename=filename,
     )
+
+    ifig = 2
+    if save_to is not None:
+        import matplotlib.pyplot as plt
+
+        figname = os.path.join(save_to, Path(filename).stem)
+        plt.savefig(f"{figname}_{ifig}.pdf")
+
+    ifig += 1
+
     if mf is None:
         return None
 
@@ -1156,7 +1261,7 @@ def extract_bs(
 
     # 4. Compute indices for the multiple triangle technique (tri_pix function)
     # -------------------------------------------------------------------------
-    l_B = np.sqrt(mf.u ** 2 + mf.v ** 2)  # Length of different bl [m]
+    l_B = np.sqrt(mf.u**2 + mf.v**2)  # Length of different bl [m]
     minbl = np.min(l_B)
 
     if n_holes >= 15:
@@ -1174,10 +1279,17 @@ def extract_bs(
     # ------------------------------------------------------------------------
     if display:
         _show_complex_ps(ft_arr)
+        if save_to is not None:
+            plt.savefig(f"{figname}_{ifig}.pdf")
+        ifig += 1
+
         _show_peak_position(ft_arr, n_baselines, mf, maskname, peakmethod)
+        if save_to is not None:
+            plt.savefig(f"{figname}_{ifig}.pdf")
+        ifig += 1
 
     if verbose:
-        print("\nFilename: %s" % filename.split("/")[-1])
+        print("\nFilename: %s" % filename)
         print("# of frames = %i" % n_ps)
 
     n_blocks = _set_good_nblocks(n_blocks, n_ps)
@@ -1198,6 +1310,7 @@ def extract_bs(
         dark_ps=None,
         closing_tri_pix=closing_tri_pix,
         bs_multi_tri=bs_multi_tri,
+        verbose=verbose,
     )
 
     cvis_arr = complex_bs["vis_arr"]["complex"]
@@ -1228,7 +1341,9 @@ def extract_bs(
     )
 
     if compute_cp_cov:
-        cp_cov = _compute_cp_cov(bs_arr, bs_quantities["bs"], index_mask)
+        cp_cov = _compute_cp_cov(
+            bs_arr, bs_quantities["bs"], index_mask, disable=np.invert(verbose)
+        )
     else:
         cp_cov = None
 
@@ -1242,18 +1357,29 @@ def extract_bs(
         fluxes,
         index_mask,
         infos,
-        expert_plot=expert_plot,
+        expert_plot=display,
     )
+    if save_to is not None:
+        plt.savefig(f"{figname}_{ifig}.pdf")
+    ifig += 1
+
     obs_result["vis2"] = vis2_norm
 
+    # 10. Now we compute the cp quantities and store them with the other observables
+    obs_result = _compute_cp(obs_result, obs_norm, infos, expert_plot=display)
+
+    if save_to is not None:
+        plt.savefig(f"{figname}_{ifig}.pdf")
+    ifig += 1
+
     if display:
         _show_norm_matrices(obs_norm, expert_plot=expert_plot)
-
-    # 10. Now we compute the cp quantities and store them with the other observables
-    obs_result = _compute_cp(obs_result, obs_norm, infos, expert_plot=expert_plot)
+        if save_to is not None:
+            plt.savefig(f"{figname}_{ifig}.pdf")
+        ifig += 1
 
     t3_coord, bl_cp = _compute_t3_coord(mf, index_mask)
-    bl_v2 = np.sqrt(mf.u ** 2 + mf.v ** 2)
+    bl_v2 = np.sqrt(mf.u**2 + mf.v**2)
     obs_result["bl"] = bl_v2
     obs_result["bl_cp"] = bl_cp
 
@@ -1268,7 +1394,7 @@ def extract_bs(
 
     # 13. Compute the absolute oriention (North-up, East-left)
     # ------------------------------------------------------------------------
-    pa = compute_pa(hdr, n_ps, display=display, verbose=verbose)
+    pa = compute_pa(hdr, n_ps, display=display, verbose=verbose, sci_hdr=sci_hdr)
 
     # Compile informations in the storage infos class
     infos = _add_infos_header(infos, hdr, mf, pa, filename, maskname, npix)
@@ -1292,6 +1418,9 @@ def extract_bs(
     t = time.time() - start_time
     m = t // 60
 
+    if save_to is not None:
+        produce_result_pdf(save_to, Path(filename).stem)
+
     if verbose:
         cprint("\nDone (exec time: %d min %2.1f s)." % (m, t - m * 60), color="magenta")
     return dict2class(obs_result)
