@@ -9,18 +9,10 @@ OIFITS related function.
 
 --------------------------------------------------------------------
 """
-
 import datetime
 import os
 
 import numpy as np
-from astropy import units as u
-from astropy.coordinates import SkyCoord
-from astropy.io import fits
-from astropy.time import Time
-from astroquery.simbad import Simbad
-from matplotlib import pyplot as plt
-from munch import munchify as dict2class
 from termcolor import cprint
 
 from amical.tools import rad2mas
@@ -77,6 +69,8 @@ def _format_staindex_t3(tab):
 def _apply_flag(dict_calibrated, unit="arcsec"):
     """Apply flag and convert to appropriete units."""
 
+    from munch import munchify as dict2class
+
     wl = dict_calibrated["OI_WAVELENGTH"]["EFF_WAVE"]
     uv_scale = {
         "m": 1,
@@ -96,7 +90,7 @@ def _apply_flag(dict_calibrated, unit="arcsec"):
     cp = dict_calibrated["OI_T3"]["T3PHI"][flag_cp]
     e_cp = dict_calibrated["OI_T3"]["T3PHIERR"][flag_cp]
     sp_freq_cp = dict_calibrated["OI_T3"]["BL"][flag_cp] * uv_scale[unit]
-    bmax = 1.2 * np.max(np.sqrt(U ** 2 + V ** 2))
+    bmax = 1.2 * np.max(np.sqrt(U**2 + V**2))
 
     cal_flagged = dict2class(
         {
@@ -117,6 +111,45 @@ def _apply_flag(dict_calibrated, unit="arcsec"):
     return cal_flagged
 
 
+def wrap_raw(bs):
+    """
+    Wrap extraction product to save it as oifits
+
+    `bs` : {munch.Munch}
+        Object returned by amical.extract_bs() with raw observables,\n
+
+    Returns
+    --------
+    `fake_cal` : {munch.Munch}
+        Object that stores the raw observables in a format compatible with the
+        output from amical.calibrate() and the input for `amical.save()`,\n
+    """
+    from munch import munchify as dict2class
+
+    u1 = bs.u[bs.mask.bs2bl_ix[0, :]]
+    v1 = bs.v[bs.mask.bs2bl_ix[0, :]]
+    u2 = bs.u[bs.mask.bs2bl_ix[1, :]]
+    v2 = bs.v[bs.mask.bs2bl_ix[1, :]]
+
+    fake_cal = {
+        "vis2": bs.vis2,
+        "e_vis2": bs.e_vis2,
+        "cp": bs.cp,
+        "e_cp": bs.e_cp,
+        "u": bs.u,
+        "v": bs.v,
+        "wl": bs.wl,
+        "u1": u1,
+        "v1": v1,
+        "u2": u2,
+        "v2": v2,
+        "raw_t": bs,
+        "raw_c": bs,
+    }
+
+    return dict2class(fake_cal)
+
+
 def cal2dict(
     cal,
     target=None,
@@ -125,7 +158,6 @@ def cal2dict(
     snr=4,
     true_flag_v2=True,
     true_flag_t3=False,
-    include_vis=False,
     oriented=-1,
     ind_hole=None,
 ):
@@ -151,9 +183,6 @@ def cal2dict(
     `true_flag_t3` : {bool}, (optional)
         If True, real flag are computed for cp using snr threshold
         (default 4), by default True,\n
-    `include_vis` : {bool}, (optional)
-        If True, include visibility amplitude in the oifits dictionnary,
-         by default False,\n
     `oriented` {float}:
         If oriented == -1, east assumed to the left in the image, otherwise
         oriented == 1 (east to the right); (Default -1),\n
@@ -163,6 +192,8 @@ def cal2dict(
     `dic`: {dict}
         Dictionnary format of the data to be save as oifits.
     """
+    from astropy.time import Time
+
     res_t = cal.raw_t
     res_c = cal.raw_c
     n_baselines = res_t.mask.n_baselines
@@ -230,7 +261,7 @@ def cal2dict(
             "TIME": 0,
             "TARGET_ID": 1,
             "FLAG": flagV2,
-            "BL": (u1 ** 2 + v1 ** 2) ** 0.5,
+            "BL": (u1**2 + v1**2) ** 0.5,
         },
         "OI_T3": {
             "MJD": t.mjd,
@@ -266,29 +297,10 @@ def cal2dict(
             "SEEING": res_t.infos.seeing,
         },
     }
-
-    if include_vis:
-        dic["OI_VIS"] = {
-            "TARGET_ID": 1,
-            "TIME": 0,
-            "MJD": t.mjd,
-            "INT_TIME": exp_time,
-            "VISAMP": cal.visamp,
-            "VISAMPERR": cal.e_visamp,
-            "VISPHI": cal.visphi,
-            "VISPHIERR": cal.e_visphi,
-            "UCOORD": u1,
-            "VCOORD": v1,
-            "STA_INDEX": sta_index_v2,
-            "FLAG": flagV2,
-        }
     return dic
 
 
-def load(
-    filename,
-    filtname=None,
-):
+def load(filename, filtname=None):
     """Load an oifits file format and store it as dictionnary. The different keys are
     representative of the oifits standard structure ('OI_WAVELENGTH', 'OI_VIS2', etc.)
 
@@ -304,6 +316,8 @@ def load(
     dic {dict}:
         Dictionnary containing the oifits file data.
     """
+    from astropy.io import fits
+
     fitsHandler = fits.open(filename)
     hdr = fitsHandler[0].header
 
@@ -393,6 +407,8 @@ def load(
             except KeyError:
                 dic["OI_T3"]["BL"] = bl_cp
 
+    fitsHandler.close()
+
     dic["info"] = {h: hdr[h] for h in hdr}  # {'MJD': mjd,
 
     if "FILT" not in list(dic["info"].keys()):
@@ -405,6 +421,8 @@ def load(
 
 def loadc(filename):
     """Same as load but provide an easy usable output as a class format (output.v2, or output.cp)."""
+    from munch import munchify as dict2class
+
     dic = load(filename)
     res = {}
     # Extract infos
@@ -442,52 +460,58 @@ def loadc(filename):
 
 
 def save(
-    input_calibrated,
+    observables,
     oifits_file=None,
     datadir="Saveoifits",
     pa=0,
     ind_hole=None,
     fake_obj=False,
-    include_vis=False,
     true_flag_v2=True,
     true_flag_t3=False,
     snr=4,
     verbose=False,
+    *,
+    origin=None,
+    raw=False,
 ):
     """
     Summary:
     --------
 
     Save the class object (from calibrate function) into oifits format. The input
-    input_calibrated can be a list of object for IFU data (e.g.: IFS-SPHERE).
+    observables can be a list of object for IFU data (e.g.: IFS-SPHERE).
 
     Parameters:
     -----------
 
-    `input_calibrated` {class}:
-        Class or list of class containing all calibrated interferometric variable extracted
-        using calibrate (amical.calibration) function,\n
+    `observables` {class}:
+        Class or list of class containing all calibrated interferometric variable
+        extracted using calibrate (amical.calibration) function,\n
+    `oifits_file` {str}:
+        Name of the oifits file,\n
+    `datadir` {str}:
+        Folder name save the oifits files,\n
+    `pa` {float}:
+        Position angle of the observation (i.e.: north direction) [deg],\n
     `ind_hole` {int}:
         By default, ind_hole is None, all the CP are considered ncp = N(N-1)(N-2)/6. If
         ind_hole is set, save only the independant CP including the given hole
         ncp = (N-1)(N-2)/2.\n
-    `oifits_file` {str}:
-        Name of the oifits file,\n
-    `include_vis` {bool}:
-        If True, include OI_VIS table in the oifits,\n
     `fake_obj` {bool}:
         If True, observable are extracted from simulated data and so doesn't
         contain real target informations (simbad search is ignored),\n
-    `pa` {float}:
-        Position angle of the observation (i.e.: north direction) [deg],\n
     `true_flag_v2`, `true_flag_t3` {bool}:
         if True, the true flag are used using snr,\n
     `snr` {float}:
         Limit snr used to compute flags (default=4),\n
-    `datadir` {str}:
-        Folder name save the oifits files,\n
     `verbose` {bool}:
-        If True, print useful informations.
+        If True, print useful informations,
+    `origin` {str}:
+        String to use as ORIGIN key in oifits. 'Sydney University' is used if origin is
+        `None` (default=None),\n
+    `raw` {bool}:
+        Set to True if the input is not calibrated. This will only silence the warning
+        shown otherwise when an uncalibrated input is detected (default=False).\n
 
     Returns:
     --------
@@ -497,8 +521,10 @@ def save(
         Name of the saved oifits file.
 
     """
+    from astroquery.simbad import Simbad
+    from astropy.io import fits
 
-    if input_calibrated is None:
+    if observables is None:
         cprint("\nError save : Wrong data format!", on_color="on_red")
         return None
 
@@ -506,15 +532,29 @@ def save(
         print("Error: oifits filename is not given, please specify oifits_file.")
         return None
 
-    if type(input_calibrated) is not list:
-        input_calibrated = [input_calibrated]
+    if type(observables) is not list:
+        observables = [observables]
+
+    if not isinstance(origin, (str, type(None))):
+        raise TypeError("origin should be a str or None")
 
     l_dic = []
-    for ical in input_calibrated:
+    for iobs in observables:
+        # If keys correspond to raw observables, make format compatible with calibrated
+        if "cp" in iobs and "raw_t" not in iobs:
+            if not raw and verbose:
+                msg = (
+                    "The input seems to contain uncalibrated observables."
+                    " Saving raw observables as oifits is provided only for"
+                    " convenience. To re-use the observables in amical.calibrate(),"
+                    " they should be saved to a pickle file. Use raw=True to turn this"
+                    " warning off."
+                )
+                cprint(f"Warning: {msg}", "green")
+            iobs = wrap_raw(iobs)
         idic = cal2dict(
-            ical,
+            iobs,
             pa=pa,
-            include_vis=include_vis,
             true_flag_v2=true_flag_v2,
             true_flag_t3=true_flag_t3,
             ind_hole=ind_hole,
@@ -541,11 +581,11 @@ def save(
     hdu.header["DATE"] = datetime.datetime.now().strftime(
         format="%F"
     )  # , 'Creation date'
-    hdu.header["ORIGIN"] = "Sydney University"
+    hdu.header["ORIGIN"] = origin or hdr.get("ORIGIN", "Sydney University")
     hdu.header["CONTENT"] = "OIFITS2"
     hdu.header["DATE-OBS"] = hdr.get("date-obs", "")
-    hdu.header["TELESCOP"] = hdr.get("TELESCOP", "")
-    hdu.header["INSTRUME"] = hdr.get("INSTRUME", "")
+    hdu.header["TELESCOP"] = hdr.get("telescop", "")
+    hdu.header["INSTRUME"] = hdr.get("INSTRUME", dic["info"]["INSTRUME"])
     hdu.header["OBSERVER"] = hdr.get("OBSERVER", "")
     hdu.header["OBJECT"] = hdr.get("OBJECT", dic["info"]["TARGET"])
     hdu.header["INSMODE"] = "NRM"
@@ -609,21 +649,31 @@ def save(
         ra = pmra = dec = pmdec = plx = [0]
         spectyp = ["fake"]
     else:
-        try:
-            query = customSimbad.query_object(name_star)
-            coord = SkyCoord(
-                query["RA"][0] + " " + query["DEC"][0], unit=(u.hourangle, u.deg)
-            )
-            ra = [coord.ra.deg]
-            dec = [coord.dec.deg]
-            spectyp = query["SP_TYPE"]
-            pmra = query["PMRA"]
-            pmdec = query["PMDEC"]
-            plx = query["PLX_VALUE"]
-        except Exception:
+        if (name_star is not None) & (name_star != "Unknown"):
+            from astropy import units as u
+            from astropy.coordinates import SkyCoord
+
+            try:
+                query = customSimbad.query_object(name_star)
+                coord = SkyCoord(
+                    query["RA"][0] + " " + query["DEC"][0], unit=(u.hourangle, u.deg)
+                )
+                ra = [coord.ra.deg]
+                dec = [coord.dec.deg]
+                spectyp = query["SP_TYPE"]
+                pmra = query["PMRA"]
+                pmdec = query["PMDEC"]
+                plx = query["PLX_VALUE"]
+            except Exception:
+                ra = pmra = dec = pmdec = plx = [0]
+                spectyp = ["fake"]
+        else:
             ra = pmra = dec = pmdec = plx = [0]
             spectyp = ["fake"]
 
+    if name_star == "":
+        name_star = "Unknown"
+
     hdu = fits.BinTableHDU.from_columns(
         fits.ColDefs(
             (
@@ -703,79 +753,6 @@ def save(
     hdu.header["OI_REVN"] = 2, "Revision number of the table definition"
     hdulist.append(hdu)
 
-    # ------------------------------
-    #           OI VIS
-    # ------------------------------
-    if include_vis:
-        if verbose:
-            print("-> Including OI Vis table...")
-
-        data = dic["OI_VIS"]
-        if type(data["TARGET_ID"]) is int:
-            npts = len(dic["OI_VIS"]["VISAMP"])
-        else:
-            npts = 1
-
-        staIndex = _format_staindex_v2(data["STA_INDEX"])
-        if type(data["MJD"]) is not float:
-            mjd = data["MJD"][0]
-        else:
-            mjd = data["MJD"]
-
-        hdu = fits.BinTableHDU.from_columns(
-            fits.ColDefs(
-                [
-                    fits.Column(
-                        name="TARGET_ID", format="1I", array=[data["TARGET_ID"]] * npts
-                    ),
-                    fits.Column(
-                        name="TIME",
-                        format="1D",
-                        unit="SECONDS",
-                        array=[data["TIME"]] * npts,
-                    ),
-                    fits.Column(
-                        name="MJD", unit="DAY", format="1D", array=[data["MJD"]] * npts
-                    ),
-                    fits.Column(
-                        name="INT_TIME",
-                        format="1D",
-                        unit="SECONDS",
-                        array=[data["INT_TIME"]] * npts,
-                    ),
-                    fits.Column(name="VISAMP", format="1D", array=data["VISAMP"]),
-                    fits.Column(name="VISAMPERR", format="1D", array=data["VISAMPERR"]),
-                    fits.Column(
-                        name="VISPHI",
-                        format="1D",
-                        unit="DEGREES",
-                        array=np.rad2deg(data["VISPHI"]),
-                    ),
-                    fits.Column(
-                        name="VISPHIERR",
-                        format="1D",
-                        unit="DEGREES",
-                        array=np.rad2deg(data["VISPHIERR"]),
-                    ),
-                    fits.Column(
-                        name="UCOORD", format="1D", unit="METERS", array=data["UCOORD"]
-                    ),
-                    fits.Column(
-                        name="VCOORD", format="1D", unit="METERS", array=data["VCOORD"]
-                    ),
-                    fits.Column(name="STA_INDEX", format="2I", array=staIndex),
-                    fits.Column(name="FLAG", format="1L", array=data["FLAG"]),
-                ]
-            )
-        )
-
-        hdu.header["OI_REVN"] = 2, "Revision number of the table definition"
-        hdu.header["EXTNAME"] = "OI_VIS"
-        hdu.header["INSNAME"] = dic["info"]["INSTRUME"]
-        hdu.header["ARRNAME"] = dic["info"]["MASK"]
-        hdu.header["DATE-OBS"] = hdr.get("date-obs", "")
-        hdulist.append(hdu)
-
     # ------------------------------
     #           OI VIS2
     # ------------------------------
@@ -947,11 +924,11 @@ def _plot_UV(ax1, l_dic, dic_color, diffWl=False):
         V = tmp.V
         band = tmp.band
         wl = tmp.wl
-        label = fr"{wl * 1e6:2.2f} $\mu m$ ({band})"
+        label = rf"{wl * 1e6:2.2f} $\mu m$ ({band})"
         if diffWl:
             c1, c2 = dic_color[band], dic_color[band]
             if band not in l_band_al:
-                label = fr"{wl * 1e6:2.2f} $\mu m$ ({band})"
+                label = rf"{wl * 1e6:2.2f} $\mu m$ ({band})"
                 l_band_al.append(band)
             else:
                 label = ""
@@ -974,54 +951,6 @@ def _plot_UV(ax1, l_dic, dic_color, diffWl=False):
     return l_bmax
 
 
-def _plot_UV_ifu(ax1, fig, l_dic):
-    l_bmax = []
-    all_U, all_V, all_wl = [], [], []
-
-    for dic in l_dic:
-        tmp = _apply_flag(dic)
-        U = tmp.U
-        V = tmp.V
-        wl = tmp.wl
-        l_bmax.append(tmp.bmax)
-        all_U.append(U)
-        all_V.append(V)
-        all_wl.append([wl] * len(U))
-    all_U = np.array(all_U)
-    all_V = np.array(all_V)
-    all_wl = np.array(all_wl) * 1e6
-
-    ax1.scatter(
-        all_U,
-        all_V,
-        s=40,
-        c=all_wl,
-        marker="o",
-        edgecolors="#364f6b",
-        alpha=1,
-        linewidth=0.1,
-        cmap="jet",
-    )
-    sc = ax1.scatter(
-        -all_U,
-        -all_V,
-        s=40,
-        c=all_wl,
-        marker="o",
-        edgecolors="#364f6b",
-        alpha=1,
-        linewidth=0.1,
-        cmap="jet",
-    )
-
-    position = fig.add_axes([0.22, 0.95, 0.1, 0.015])
-    fig.colorbar(sc, cax=position, orientation="horizontal", drawedges=False)
-    ax1.text(
-        0.53, 0.98, r"$\lambda$ [µm]", ha="center", va="center", transform=ax1.transAxes
-    )
-    return l_bmax
-
-
 def _plot_V2(ax2, l_dic, dic_color, diffWl=False):
     max_f_vis = []
     for dic in l_dic:
@@ -1053,55 +982,6 @@ def _plot_V2(ax2, l_dic, dic_color, diffWl=False):
     return max_f_vis
 
 
-def _plot_V2_ifu(ax2, l_dic):
-    max_f_vis = []
-    all_V2, all_e_V2, all_freq, all_wl = [], [], [], []
-    for dic in l_dic:
-        tmp = _apply_flag(dic, unit="arcsec")
-        V2 = tmp.vis2
-        e_V2 = tmp.e_vis2
-        sp_freq_vis = tmp.sp_freq_vis
-        max_f_vis.append(np.max(sp_freq_vis))
-
-        all_V2.extend(V2)
-        all_e_V2.extend(e_V2)
-        all_freq.extend(sp_freq_vis)
-        all_wl.extend([tmp.wl] * len(V2))
-
-    all_V2 = np.array(all_V2)
-    all_e_V2 = np.array(all_e_V2)
-    all_freq = np.array(all_freq)
-    all_wl = np.array(all_wl)
-
-    ax2.errorbar(
-        all_freq,
-        all_V2,
-        yerr=all_e_V2,
-        linestyle="None",
-        capsize=1,
-        ecolor="#364f6b",
-        mec="#364f6b",
-        marker="None",
-        elinewidth=0.5,
-        alpha=1,
-        ms=9,
-    )
-    ax2.scatter(
-        all_freq,
-        all_V2,
-        s=20,
-        c=all_wl,
-        zorder=20,
-        linewidth=0.5,
-        marker="o",
-        edgecolors="#364f6b",
-        alpha=1,
-        cmap="jet",
-    )
-
-    return max_f_vis
-
-
 def _plot_CP(ax3, l_dic, dic_color, conv_cp, diffWl=False):
     max_f_cp = []
     for dic in l_dic:
@@ -1133,6 +1013,103 @@ def _plot_CP(ax3, l_dic, dic_color, conv_cp, diffWl=False):
     return max_f_cp
 
 
+def _plot_UV_ifu(ax1, fig, l_dic):
+    l_bmax = []
+    all_U, all_V, all_wl = [], [], []
+
+    for dic in l_dic:
+        tmp = _apply_flag(dic)
+        U = tmp.U
+        V = tmp.V
+        wl = tmp.wl
+        l_bmax.append(tmp.bmax)
+        all_U.append(U)
+        all_V.append(V)
+        all_wl.append([wl] * len(U))
+    all_U = np.array(all_U)
+    all_V = np.array(all_V)
+    all_wl = np.array(all_wl) * 1e6
+
+    ax1.scatter(
+        all_U,
+        all_V,
+        s=40,
+        c=all_wl,
+        marker="o",
+        edgecolors="#364f6b",
+        alpha=1,
+        linewidth=0.1,
+        cmap="turbo",
+    )
+    sc = ax1.scatter(
+        -all_U,
+        -all_V,
+        s=40,
+        c=all_wl,
+        marker="o",
+        edgecolors="#364f6b",
+        alpha=1,
+        linewidth=0.1,
+        cmap="turbo",
+    )
+
+    position = fig.add_axes([0.22, 0.95, 0.1, 0.015])
+    fig.colorbar(sc, cax=position, orientation="horizontal", drawedges=False)
+    ax1.text(
+        0.53, 0.98, r"$\lambda$ [µm]", ha="center", va="center", transform=ax1.transAxes
+    )
+    return l_bmax
+
+
+def _plot_V2_ifu(ax2, l_dic):
+    max_f_vis = []
+    all_V2, all_e_V2, all_freq, all_wl = [], [], [], []
+    for dic in l_dic:
+        tmp = _apply_flag(dic, unit="arcsec")
+        V2 = tmp.vis2
+        e_V2 = tmp.e_vis2
+        sp_freq_vis = tmp.sp_freq_vis
+        max_f_vis.append(np.max(sp_freq_vis))
+
+        all_V2.extend(V2)
+        all_e_V2.extend(e_V2)
+        all_freq.extend(sp_freq_vis)
+        all_wl.extend([tmp.wl] * len(V2))
+
+    all_V2 = np.array(all_V2)
+    all_e_V2 = np.array(all_e_V2)
+    all_freq = np.array(all_freq)
+    all_wl = np.array(all_wl)
+
+    ax2.errorbar(
+        all_freq,
+        all_V2,
+        yerr=all_e_V2,
+        linestyle=None,
+        capsize=1,
+        ecolor="#364f6b",
+        mec="#364f6b",
+        marker="None",
+        elinewidth=0.5,
+        alpha=1,
+        ms=9,
+    )
+    ax2.scatter(
+        all_freq,
+        all_V2,
+        s=20,
+        c=all_wl,
+        zorder=20,
+        linewidth=0.5,
+        marker="o",
+        edgecolors="#364f6b",
+        alpha=1,
+        cmap="turbo",
+    )
+
+    return max_f_vis
+
+
 def _plot_CP_ifu(ax3, l_dic, conv_cp):
     max_f_cp = []
     all_CP, all_e_cp, all_freq, all_wl = [], [], [], []
@@ -1171,7 +1148,7 @@ def _plot_CP_ifu(ax3, l_dic, conv_cp):
         marker="o",
         edgecolors="#364f6b",
         alpha=1,
-        cmap="jet",
+        cmap="turbo",
     )
     return max_f_cp
 
@@ -1220,6 +1197,7 @@ def show(
     `snr` {float}:
         If inputs are classes from amical.calibrate, use snr param to compute flag,
     """
+    import matplotlib.pyplot as plt
 
     if type(inputList) is not list:
         inputList = [inputList]
@@ -1262,6 +1240,10 @@ def show(
             dic_color[filt] = list_color[i_c]
             i_c += 1
 
+    props = dict(boxstyle="round", facecolor="wheat", alpha=0.5)
+
+    textstr = "PA = %2.1f deg" % pa
+
     fontsize = 14
     fig = plt.figure(figsize=(16, 5.5))
     ax1 = plt.subplot2grid((2, 6), (0, 0), rowspan=2, colspan=2)
@@ -1271,8 +1253,7 @@ def show(
     # Plot plan UV
     # -------
     ins = l_dic[0]["info"]["INSTRUME"]
-
-    if ("IFS" in ins) & (len(l_dic) > 1):
+    if "IFS" in ins and len(l_dic) > 1:
         l_bmax = _plot_UV_ifu(ax1, fig, l_dic)
     else:
         l_bmax = _plot_UV(ax1, l_dic, dic_color, diffWl=False)
@@ -1292,6 +1273,16 @@ def show(
         labels, handles = zip(*sorted(zip(labels, handles), key=lambda t: t[0]))
         ax1.legend(handles, labels, loc="best", fontsize=9)
 
+    plt.text(
+        0.02,
+        0.98,
+        textstr,
+        transform=ax1.transAxes,
+        fontsize=13,
+        verticalalignment="top",
+        bbox=props,
+    )
+
     unitlabel = {
         "m": "m",
         "rad": "rad$^{-1}$",
@@ -1305,7 +1296,7 @@ def show(
 
     # Plot V2
     # -------
-    if ("IFS" in ins) & (len(l_dic) > 1):
+    if "IFS" in ins and len(l_dic) > 1:
         max_f_vis = _plot_V2_ifu(ax2, l_dic)
     else:
         max_f_vis = _plot_V2(ax2, l_dic, dic_color, diffWl=diffWl)
@@ -1339,7 +1330,7 @@ def show(
 
     cmin = -cmax
 
-    if ("IFS" in ins) & (len(l_dic) > 1):
+    if "IFS" in ins and len(l_dic) > 1:
         max_f_cp = _plot_CP_ifu(ax3, l_dic, conv_cp)
     else:
         max_f_cp = _plot_CP(ax3, l_dic, dic_color, conv_cp, diffWl=diffWl)
