@@ -9,23 +9,11 @@ General tools.
 
 --------------------------------------------------------------------
 """
-
 import math as m
 import warnings
 
-import h5py
-import matplotlib.pyplot as plt
 import numpy as np
-from astropy.io import fits
-from astropy.nddata import Cutout2D
-from astropy.time import Time
-from munch import munchify as dict2class
-from scipy.signal import medfilt2d
 from termcolor import cprint
-from uncertainties import ufloat
-
-warnings.filterwarnings("ignore", module="astropy.io.votable.tree")
-warnings.filterwarnings("ignore", module="astropy.io.votable.xmlutil")
 
 
 def linear(x, param):
@@ -44,10 +32,50 @@ def mas2rad(mas):
 
 def rad2mas(rad):
     """Convert input angle in radians to milli-arcsec"""
-    mas = rad * (3600.0 * 180 / np.pi) * 10.0 ** 3
+    mas = rad * (3600.0 * 180 / np.pi) * 10.0**3
     return mas
 
 
+def find_max(img, filtmed=True, f=3):
+    """
+    Summary
+    -------------
+    Find brightest pixel of an image
+
+    Parameters
+    ----------
+    `img` : {numpy.array}
+        input image,\n
+    `filtmed` : {boolean}, (optionnal)
+        True if perform a median filter on the image (to blur bad pixels),\n
+    `f` : {float}, (optionnal),
+        If filtmed == True, kernel size of the median filter.
+
+
+    Returns
+    -------
+    `Max coordinates`: {tuple}
+        X and Y positions of max pixel
+    """
+    from scipy.signal import medfilt2d
+
+    if filtmed:
+        try:
+            im_med = medfilt2d(img, f)
+        except ValueError:
+            img = img.astype(float)
+            im_med = medfilt2d(img, f)
+    else:
+        im_med = img.copy()
+
+    pos_max = np.where(im_med == im_med.max())
+
+    X = pos_max[1][0]
+    Y = pos_max[0][0]
+
+    return X, Y
+
+
 def crop_max(img, dim, offx=0, offy=0, filtmed=True, f=3):
     """
     Summary
@@ -71,18 +99,20 @@ def crop_max(img, dim, offx=0, offy=0, filtmed=True, f=3):
     `cutout`: {numpy.array}
         Resized image.
     """
-    if filtmed:
-        try:
-            im_med = medfilt2d(img, f)
-        except ValueError:
-            img = img.astype(float)
-            im_med = medfilt2d(img, f)
-    else:
-        im_med = img.copy()
+    from astropy.nddata import Cutout2D
 
-    pos_max = np.where(im_med == im_med.max())
-    X = pos_max[1][0] + offx
-    Y = pos_max[0][0] + offy
+    xmax, ymax = find_max(img, filtmed=filtmed, f=f)
+
+    X = xmax + offx
+    Y = ymax + offy
+    isz_max = 2 * np.min([X, img.shape[1] - X - 1, Y, img.shape[0] - Y - 1]) + 1
+    if isz_max < dim:
+        size_msg = (
+            f"The specified cropped image size, {dim}, is greater than the distance to"
+            " the PSF center in at least one dimension. The max size for this image is"
+            f" {isz_max}"
+        )
+        raise ValueError(size_msg)
     cutout = Cutout2D(img, (X, Y), dim)
     return cutout.data, (X, Y)
 
@@ -107,55 +137,6 @@ def norm_max(tab):
     return tab_norm
 
 
-def crop_center(img, dim):
-    """
-    Short Summary
-    -------------
-    Resize an image on the center.
-
-    Parameters
-    ----------
-    `img` : {numpy.array}
-        input image,\n
-    `dim` : {int}
-        resized dimension.
-
-    Returns
-    -------
-    `cutout`: {numpy.array}
-        Resized image.
-    """
-    b = img.shape[0]
-    position = (b // 2, b // 2)
-    cutout = Cutout2D(img, position, dim)
-    return cutout.data
-
-
-def crop_position(img, X, Y, dim):
-    """
-    Short Summary
-    -------------
-    Resize an image on a defined position.
-
-    Parameters
-    ----------
-    `img` : {numpy.array}
-        input image,\n
-    `X`, `Y` : {int}
-        Position to resize (new center of the image),\n
-    `dim` : {int}
-        resized dimension.
-
-    Returns
-    -------
-    `cutout`: {numpy.array}
-        Resized image.
-    """
-    position = (X, Y)
-    cutout = Cutout2D(img, position, dim)
-    return cutout.data
-
-
 def gauss_2d_asym(X, param):
     """
     Short Summary
@@ -207,14 +188,14 @@ def gauss_2d_asym(X, param):
     im = np.zeros([size_y, size_x])
     x0 = float(x0)
     y0 = float(y0)
-    a = (np.cos(theta) ** 2) / (2 * sigma_x ** 2) + (np.sin(theta) ** 2) / (
-        2 * sigma_y ** 2
+    a = (np.cos(theta) ** 2) / (2 * sigma_x**2) + (np.sin(theta) ** 2) / (
+        2 * sigma_y**2
     )
-    b = -(np.sin(2 * theta)) / (4 * sigma_x ** 2) + (np.sin(2 * theta)) / (
-        4 * sigma_y ** 2
+    b = -(np.sin(2 * theta)) / (4 * sigma_x**2) + (np.sin(2 * theta)) / (
+        4 * sigma_y**2
     )
-    c = (np.sin(theta) ** 2) / (2 * sigma_x ** 2) + (np.cos(theta) ** 2) / (
-        2 * sigma_y ** 2
+    c = (np.sin(theta) ** 2) / (2 * sigma_x**2) + (np.cos(theta) ** 2) / (
+        2 * sigma_y**2
     )
     im = amplitude * np.exp(
         -(a * ((x - x0) ** 2) + 2 * b * (x - x0) * (y - y0) + c * ((y - y0) ** 2))
@@ -222,17 +203,6 @@ def gauss_2d_asym(X, param):
     return im
 
 
-def conv_fft(image, psf):
-    """
-    Compute 2D convolution with the PSF, passing through Fourier space.
-    """
-    fft_im = np.fft.fft2(image)
-    fft_psf = np.fft.fft2(psf)
-    fft_conv = fft_im * fft_psf
-    conv = abs(np.fft.fftshift(np.fft.ifft2(fft_conv)))
-    return conv
-
-
 def plot_circle(d, x, y, hole_radius, sz=1, display=True):
     """Return an image with a disk = sz at x, y position and zero elsewhere"""
     chipsz = np.shape(d)[0]
@@ -264,6 +234,8 @@ def plot_circle(d, x, y, hole_radius, sz=1, display=True):
                     im[i, j] = sz
 
     if display:
+        import matplotlib.pyplot as plt
+
         plt.figure()
         plt.imshow(im)
 
@@ -303,40 +275,6 @@ def cov2cor(cov):
     return cor, sigma
 
 
-def sky_correction(imA, r1=100, dr=20, verbose=False):
-    """
-    Perform background sky correction to be as close to zero as possible.
-    """
-    isz = imA.shape[0]
-    xc, yc = isz // 2, isz // 2
-    xx, yy = np.arange(isz), np.arange(isz)
-    xx2 = xx - xc
-    yy2 = yc - yy
-    r2 = r1 + dr
-
-    distance = np.sqrt(xx2 ** 2 + yy2[:, np.newaxis] ** 2)
-    cond_bg = (r1 <= distance) & (distance <= r2)
-
-    try:
-        minA = imA.min()
-        imB = imA + 1.01 * abs(minA)
-        backgroundB = np.mean(imB[cond_bg])
-        imC = imB - backgroundB
-        backgroundC = np.mean(imC[cond_bg])
-    except IndexError:
-        imC = imA.copy()
-        backgroundC = 0
-        cprint("Warning: Background not computed", "green")
-        cprint("-> check the inner and outer radius rings (checkrad option).", "green")
-
-    return imC, backgroundC
-
-
-def super_gauss(dist, hwhm, m):
-    y = np.exp(-np.log(0.5) * (dist / hwhm) ** m)
-    return y
-
-
 def super_gaussian(x, sigma, m, amp=1, x0=0):
     sigma = float(sigma)
     m = float(m)
@@ -358,7 +296,7 @@ def apply_windowing(img, window=80, m=3):
     xx2 = xx - isz // 2
     yy2 = isz // 2 - yy
     # Distance map
-    distance = np.sqrt(xx2 ** 2 + yy2[:, np.newaxis] ** 2)
+    distance = np.sqrt(xx2**2 + yy2[:, np.newaxis] ** 2)
 
     # Super-gaussian windowing
     window = super_gaussian(distance, sigma=window * 2, m=m)
@@ -368,13 +306,7 @@ def apply_windowing(img, window=80, m=3):
     return img_apod
 
 
-def compute_ufloat_arr(data, e_data):
-    """Compute the array containing ufloat format used by uncertainties package."""
-    u_data = np.array([ufloat(data[i], e_data[i]) for i in range(len(data))])
-    return u_data
-
-
-def sanitize_array(dic):
+def sanitize_array(dic):  # pragma: no cover
     """Recursively convert values in a nested dictionnary from np.bool_ to builtin bool type
     This is required for json serialization.
     """
@@ -423,22 +355,23 @@ def jd2lst(lng, jd):
     t = t0 / 36525.0
 
     # Compute GST in seconds.
-    theta = c[0] + (c[1] * t0) + t ** 2 * (c[2] - t / c[3])
+    theta = c[0] + (c[1] * t0) + t**2 * (c[2] - t / c[3])
 
     # Compute LST in hours.
     lst = (theta + lng) / 15.0
-    neg = np.where(lst < 0.0)
-    n = neg[0].size
+    cond_neg = lst < 0.0
+    n = lst[cond_neg].size
     if n > 0:
-        lst[neg] = 24.0 + (lst[neg] % 24)
+        lst[cond_neg] = 24.0 + (lst[cond_neg] % 24)
     lst = lst % 24
     return lst
 
 
-def compute_pa(hdr, n_ps, verbose=False, display=False):
+def compute_pa(hdr, n_ps, verbose=False, display=False, *, sci_hdr=None):
 
     list_fct_pa = {
-        "SPHERE": sphere_parang,
+        "SPHERE": (sphere_parang, {"hdr": hdr, "n_dit_ifs": n_ps}),
+        "NIRISS": (niriss_parang, {"hdr": sci_hdr}),
     }
 
     instrument = hdr["INSTRUME"]
@@ -456,15 +389,18 @@ def compute_pa(hdr, n_ps, verbose=False, display=False):
         pa_exist = False
         l_pa = np.zeros(nframe)
     else:
-        l_pa = list_fct_pa[instrument](hdr, n_ps)
+        fct_pa, kwargs_pa = list_fct_pa[instrument]
+        l_pa = fct_pa(**kwargs_pa)
         pa_exist = True
 
     pa = np.mean(l_pa)
     std_pa = np.std(l_pa)
 
     if display and pa_exist:
+        import matplotlib.pyplot as plt
+
         plt.figure(figsize=(4, 3))
-        plt.plot(l_pa, ".-", label=fr"pa={pa:2.1f}, $\sigma_{{pa}}$={std_pa:2.1f} deg")
+        plt.plot(l_pa, ".-", label=rf"pa={pa:2.1f}, $\sigma_{{pa}}$={std_pa:2.1f} deg")
         plt.legend(fontsize=7)
         plt.grid(alpha=0.2)
         plt.xlabel("# frames")
@@ -474,6 +410,19 @@ def compute_pa(hdr, n_ps, verbose=False, display=False):
     return pa
 
 
+def niriss_parang(hdr):
+    if hdr is None:
+        warnings.warn(
+            "No SCI header for NIRISS. No PA correction will be applied.",
+            RuntimeWarning,
+        )
+        return 0.0
+    v3i_yang = hdr["V3I_YANG"]  # Angle from V3 axis to ideal y axis (deg)
+    roll_ref_pa = hdr["ROLL_REF"]  # Offset between V3 and N in local aperture coord
+
+    return roll_ref_pa - v3i_yang
+
+
 def sphere_parang(hdr, n_dit_ifs=None):
     """
     Reads the header and creates an array giving the paralactic angle for each frame,
@@ -481,6 +430,7 @@ def sphere_parang(hdr, n_dit_ifs=None):
     The columns of the output array contains:
     frame_number, frame_time, paralactic_angle
     """
+    from astropy.time import Time
 
     r2d = 180 / np.pi
     d2r = np.pi / 180
@@ -636,7 +586,7 @@ def sphere_parang(hdr, n_dit_ifs=None):
     return parang_array
 
 
-def check_seeing_cond(list_nrm):
+def check_seeing_cond(list_nrm):  # pragma: no cover
     """Extract the seeing conditions, parang, averaged vis2
     and cp of a list of nrm classes extracted with extract_bs
     function (bispect.py).
@@ -647,11 +597,16 @@ def check_seeing_cond(list_nrm):
     `res.infos.seeing` for the seeing across multiple nrm data (files).
 
     """
+    from astropy.io import fits
+    from munch import munchify as dict2class
+
     l_seeing, l_vis2, l_cp, l_pa, l_mjd = [], [], [], [], []
 
-    hdr = fits.open(list_nrm[0].infos.filename)[0].header
+    with fits.open(list_nrm[0].infos.filename) as fd:
+        hdr = fd[0].header
     for nrm in list_nrm:
-        hdr = fits.open(nrm.infos.filename)[0].header
+        with fits.open(nrm.infos.filename) as fd:
+            hdr = fd[0].header
         pa = np.mean(sphere_parang(hdr))
         seeing = nrm.infos.seeing
         mjd = hdr["MJD-OBS"]
@@ -673,27 +628,9 @@ def check_seeing_cond(list_nrm):
     return dict2class(sanitize_array(res))
 
 
-def plot_seeing_cond(cond, lim_seeing=None):
+def plot_seeing_cond(cond, lim_seeing=None):  # pragma: no cover
     """Plot seeing condition between calibrator and target files."""
-
-    # l_xmin, l_xmax = [], []
-    # # for x in cond:
-
-    # #     try:
-    # #         m_mjd = abs(np.min(np.diff(x.mjd)))/2.
-    # #     except ValueError:
-
-    # #     xmin = np.min([x.mjd.min(), x.mjd.min()])-m_mjd
-    # #     xmax = np.max([x.mjd.max(), x.mjd.max()])+m_mjd
-    # #     l_xmin.append(xmin)
-    # #     l_xmax.append(xmax)
-
-    # xmin = np.min(l_xmin)
-    # xmax = np.min(l_xmax)
-
-    # m_mjd=abs(np.min(np.diff(cond_t.mjd)))/2.
-    # xmin=np.min([cond_c.mjd.min(), cond_t.mjd.min()])-m_mjd
-    # xmax=np.max([cond_c.mjd.max(), cond_t.mjd.max()])+m_mjd
+    import matplotlib.pyplot as plt
 
     fig = plt.figure()
     ax1 = plt.gca()
@@ -704,17 +641,12 @@ def plot_seeing_cond(cond, lim_seeing=None):
     ax1.set_ylabel("Uncalibrated mean V$^2$")
 
     for x in cond:
-        ax1.plot(x.mjd, x.vis2, ".", label=x.target)  # color="#20b2aa"
+        ax1.plot(x.mjd, x.vis2, ".", label=x.target)
         ax2.plot(x.mjd, x.seeing, "+", color="#c62d42")
-    # ax1.plot(cond_c.mjd, cond_c.vis2, '.',
-    #          label="%s (cal)" % cond_c.target)
-    # ax2.plot(cond_c.mjd, cond_c.seeing, '+',
-    #          color="#c62d42", label='Seeing')
     if lim_seeing is not None:
         ax2.axhline(lim_seeing, color="g", label="Seeing threshold")
     ax1.set_ylim(0, 1.2)
     ax2.set_ylim(0.6, 1.8)
-    # ax1.set_xlim(xmin, xmax)
     ax1.grid(alpha=0.1, color="grey")
     ax1.legend(loc="best", fontsize=9)
     plt.tight_layout()
@@ -741,6 +673,8 @@ def roundSciDigit(number):
 
 def save_bs_hdf5(bs, filename):
     """Save results from `amical.extract_bs()` into hdf5 file."""
+    import h5py
+
     if ".h5" not in filename:
         filename += ".h5"
 
@@ -773,20 +707,24 @@ def save_bs_hdf5(bs, filename):
     for key in mask:
         if (mask[key] is not None) and (key != "t3_coord"):
             grp_mask.create_dataset(key, data=mask[key])
+
     t3_coord = mask["t3_coord"]
     for key in t3_coord:
         grp_mask.create_dataset(key, data=t3_coord[key])
 
     # Step 5 - Save informations (target, date, etc.).
     infos = bs.infos
-    for i in infos:
-        if i != "hdr":
-            grp_info.create_dataset(i, data=infos[i])
+    for key in infos:
+        if key != "hdr":
+            info = infos[key]
+            if info is None:
+                info = "Unknown"
+            grp_info.attrs[key] = info
 
     # Step 6 - Save original header keywords.
     hdr = bs.infos.hdr
     for key in hdr:
-        grp_hdr.create_dataset(key, data=hdr[key])
+        grp_hdr.attrs[key] = str(hdr[key])
 
     # Last step - close hdf5
     hf.close()
@@ -797,42 +735,43 @@ def load_bs_hdf5(filename):
     """Load hdf5 file and format as class like object (same
     format as `amical.extract_bs()`
     """
-    hf2 = h5py.File(filename, "r")
+    from munch import munchify as dict2class
+    import h5py
 
     dict_bs = {"matrix": {}, "infos": {"hdr": {}}, "mask": {}}
+    with h5py.File(filename, "r") as hf2:
+        obs = hf2["obs"]
+        for o in obs:
+            dict_bs[o] = np.array(obs.get(o))
 
-    obs = hf2["obs"]
-    for o in obs:
-        dict_bs[o] = obs[o].value
+        matrix = hf2["matrix"]
+        for key in matrix:
+            dict_bs["matrix"][key] = np.array(matrix.get(key))
 
-    matrix = hf2["matrix"]
-    for key in matrix:
-        dict_bs["matrix"][key] = matrix[key].value
+        if len(dict_bs["matrix"]["cp_cov"]) == 1:
+            dict_bs["matrix"]["cp_cov"] = None
 
-    if len(dict_bs["matrix"]["cp_cov"]) == 1:
-        dict_bs["matrix"]["cp_cov"] = None
+        mask = hf2["mask"]
+        for key in mask:
+            if key not in ["u1", "u2", "v1", "v2"]:
+                dict_bs["mask"][key] = np.array(mask.get(key))
 
-    mask = hf2["mask"]
-    for key in mask:
-        if key not in ["u1", "u2", "v1", "v2"]:
-            dict_bs["mask"][key] = mask[key].value
+        t3_coord = {
+            "u1": np.array(mask.get("u1")),
+            "u2": np.array(mask.get("u2")),
+            "v1": np.array(mask.get("v1")),
+            "v2": np.array(mask.get("v2")),
+        }
 
-    t3_coord = {
-        "u1": mask["u1"].value,
-        "u2": mask["u2"].value,
-        "v1": mask["v1"].value,
-        "v2": mask["v2"].value,
-    }
+        dict_bs["mask"]["t3_coord"] = t3_coord
 
-    dict_bs["mask"]["t3_coord"] = t3_coord
+        infos = hf2["infos"]
+        for key in hf2["infos"].attrs.keys():
+            dict_bs["infos"][key] = infos.attrs[key]
 
-    infos = hf2["infos"]
-    for key in infos:
-        dict_bs["infos"][key] = infos[key].value
+        hdr = hf2["hdr"]
+        for key in hdr.attrs.keys():
+            dict_bs["infos"]["hdr"][key] = hdr.attrs[key]
 
-    hdr = hf2["hdr"]
-    for key in hdr:
-        dict_bs["infos"]["hdr"][key] = hdr[key].value
-
-    bs_save = dict2class(dict_bs)
+        bs_save = dict2class(dict_bs)
     return bs_save
